Package: lakhesis
Title: Consensus Seriation for Binary Data
Version: 1.1.2
Authors@R: 
    person(c("Stephen", "A."), "Collins-Elliott", , "sce@utk.edu", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-5642-6903"))
Description: Determining consensus seriations for binary incidence matrices, using a two-step process of Procrustes-fit correspondence analysis for heuristic selection of partial seriations and iterative regression to establish a single consensus. Contains the Lakhesis Calculator, a graphical platform for identifying seriated sequences. Collins-Elliott (2026) "Lakhesis: Consensus Seriation via Iterative Regression of Partial Rankings for Binary Data" <doi:10.1080/02664763.2026.2672564>.
License: GPL (>= 3)
Imports: stats, Rcpp, RcppArmadillo, readr, ca, ggplot2, Rdpack, shiny,
        shinydashboard, bslib
RdMacros: Rdpack
Encoding: UTF-8
RoxygenNote: 7.3.2
LinkingTo: Rcpp, RcppArmadillo
Depends: R (>= 3.5.0)
LazyData: true
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-05-28 17:17:18 UTC; archaeologus
Author: Stephen A. Collins-Elliott [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5642-6903>)
Maintainer: Stephen A. Collins-Elliott <sce@utk.edu>
Repository: CRAN
Date/Publication: 2026-05-28 17:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-28 21:36:33 UTC; unix
Archs: lakhesis.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(ca_procrustes,default)
S3method(ca_procrustes,incidence_matrix)
S3method(ca_procrustes,matrix)
S3method(ca_procrustes_ser,incidence_matrix)
S3method(ca_procrustes_ser,matrix)
S3method(conc_c,incidence_matrix)
S3method(conc_c,matrix)
S3method(conc_wrc,incidence_matrix)
S3method(conc_wrc,matrix)
S3method(cor_sq,incidence_matrix)
S3method(cor_sq,matrix)
S3method(element_eval,incidence_matrix)
S3method(element_eval,matrix)
S3method(im_long,incidence_matrix)
S3method(im_long,matrix)
S3method(im_merge,incidence_matrix)
S3method(im_merge,matrix)
S3method(im_ref,incidence_matrix)
S3method(im_ref,matrix)
S3method(lakhesize,default)
S3method(lakhesize,strands)
S3method(plot,incidence_matrix)
S3method(plot,lakhesis)
S3method(plot,procrustean)
S3method(plot,strand)
S3method(print,lakhesis)
S3method(print,procrustean)
S3method(print,strand)
S3method(print,strands)
S3method(spearman_sq,numeric)
S3method(strand_add,strand)
S3method(strand_create,incidence_matrix)
S3method(strand_create,matrix)
S3method(strand_extract,strands)
S3method(strand_suppress,default)
S3method(strand_suppress,strands)
S3method(strands_create,list)
S3method(summary,lakhesis)
S3method(summary,strand)
export(LC)
export(ca_procrustes)
export(ca_procrustes_ser)
export(conc_c)
export(conc_wrc)
export(cor_sq)
export(element_eval)
export(im_long)
export(im_merge)
export(im_read_csv)
export(im_ref)
export(lakhesize)
export(spearman_sq)
export(strand_add)
export(strand_create)
export(strand_extract)
export(strand_suppress)
export(strands_create)
importFrom(Rcpp,sourceCpp)
importFrom(Rdpack,reprompt)
useDynLib(lakhesis)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# <img src="man/figures/logo.png" align="right" width="150px"/> lakhesis: Consensus Seriation for Binary Data

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/lakhesis)](https://CRAN.R-project.org/package=lakhesis)
<!-- badges: end -->

The `R` package `lakhesis` provides an interactive platform and critical
measures for seriating binary data matrices through the exploration,
selection, and consensus of partially seriated sequences.

Seriation (sequencing, ordination) involves putting a set of things in
an optimal order. In archaeology, seriation can be used to establish a
chronological order of contexts and find-types on the basis of their
similarity, i.e, that things come into and go out of fashion with a peak
moment of popularity (Ihm 2005). In ecology, the distribution of a
species may occur according to a preferred environmental condition that
diminishes as that environment changes (ter Braak and Looman 1986).
There are a number of R functions and packages, especially
[`seriation`](https://github.com/mhahsler/seriation) (Hahsler, Hornik,
and Buchcta 2008) and
[`vegan`](https://CRAN.R-project.org/package=vegan) (Oksanen et al.
2024) that provide the means to seriate or ordinate matrices, especially
for frequency or count data. While binary (presence/absence) data are
often viewed as a reductive case of frequency data, they can also
present their own challenges. Moreover, not all incidence matrices (the
matrix of 0/1s that record the joint incidence or occurrence for a
row-column pairing) will be well seriated. The selection of which row
and column elements to inlcude in the input is accordingly an intrinsic
part of the task of seriation. In this respect, `lakhesis` seeks to
complement existing methods in `R`, focusing on binary data, by
providing an interactive, graphical means of selecting seriated
sequences. It relies correspondence analysis (CA), a mainstay technique
for seriation, and offers a method of Procrustes-fit CA to align scores
with an ideal reference curve. Multiple seriations can be rerun on
partial subsets, called “strands,” of the initial incidence matrix,
which are then recompiled into a single consensus seriation using an
optimality criterion. The process of harmonizing different strands of
sequential elements via iterative linear regression is called a lakhesis
technique, after the fate from ancient Greek mythology who measured the
strand of one’s life. The package relies on `Rcpp` and `RcppArmadillo`
(Eddelbuettel and Sanderson 2014; Eddelbuettel and Balamuta 2018).

While command line functions can be run in `R`, the functionality of
`lakhesis` is primarily achieved via the Lakhesis Calculator, a
graphical interface in `shiny` (Chang et al. 2024) that enables
investigators to explore datasets, select strands, and harmonize them
into a single consensus seriation. Panels in the calculator include:

- **Seriation Explorer** displays the correspondence analysis plot of a
  datase, including Procrustes-fit CA (both as they have been fit to the
  curve and their orthogonal projection along the curve). Selections can
  be made on any plot.
  - Map options. Users can choose either a **symmetric** or an
    **asymmetric** CA plot.
- **Save Strand** records the displayed plot as a partial seriation, or
  “strand” (i.e., partial with respect to the initial data matrix).
  - Strands can be sequenced according to different projections:
    - **CA1** / **CA2** Projection of scores along the first or second
      principal CA axis.
    - **Procrustes1** / **Procrustes2** Projection of scores along the
      first or second axis after Procrustes fitting.
    - **Curve** Projection along the reference curve of an ideal
      seriation.
  - **Lakhesize** Produce a consensus seriation (must have saved at
    least two strands). Constructs a consensus seriation of the selected
    strands using an iterative process of linear regression of partial
    rankings in an agglomerative fashion. The matrix plot displays the
    incidence matrix of the resulting consensus seriation, with
    optimality criteria. The agreement of the seriation in each strand
    with that of the consensus seriation as well as its criterion is
    displayed in the Diganostics panel. The function `lakhesize()`
    performs this task.
  - **Run Deviance Test** performs a goodness-of-fit test using
    deviance, treating the distribution of the row and column incidences
    with a quadratic-logistic model. The largest $p$ values of the row
    and column elements is contained in the Diganostics panel. The
    function `element_eval()` performs this task.
- **Consensus Seriation** displays the results of harmonizing selected
  partial seriations, which have been identified as “strands.” The
  process of deriving a consensus seriation entails a process of
  iterative regressions on partially seriated sequences, optimized using
  the concentration measure. The seriated incidence matrix is also
  displayed in this panel.
- **Diagnostics** show critical coefficients to determine whether
  discordant strands should be removed and/or row or column elements
  should be suppressed from consideration.
  - **Agreement** expresses whether a strand agrees with consensus
    seriation.
  - **Criteria** expresses how well seriated the strand is. Options for
    optimality criteria are those which are used in Lakhesis,
    comprising:
    - **Squared correlation coefficient** (`cor_sq`).
    - **Weighted row-column concentration** (`conc_wrc`).
  - Tabs marked **Deviance** report on the goodness-of-fit of row and
    column elements in the consensus seriation using deviance with a
    quadratic-logistic model. Higher $p$ values will indicate poorer fit
    for a particular row or column element.
- **Modify** temporarily suppress row or column elements from
  correspondence analysis. Strands which have low agreement or high
  concentration may also be deleted in this panel.

The sidebar contains the following commands:

- **Choose CSV** – data must be without a header in a two-column “long”
  format of occurring pairs of row and column elements, where the first
  column contains a row element and the second column contains a column
  element of the incidence matrix.
- **Reinitialize** resets the plots to their original, starting
  condition.
- **Replot with Selection** – upon the selection of row and column
  points from the Seriation Explorer panel, this command will perform
  and fit CA only on the selection. To return to the initial dataset,
  press the Reinitialize button. The function `ca_procrustes_ser()`
  performs this task.
- **Export Data** will download results in a single `.rds` file, which
  is a `list` class object containing the following:
  - `consensus` The results of `lakhesize()`, a `lakhesis` class object
    containing row and column consensus seriations, coefficients of
    agreement and concentration, and the seriated incidence matrix.
  - `strands` The strands selected to produce `consensus`.

## Installation

To obtain the current development version of `lakhesis` from GitHub,
install from GitHub in the `R` command line with:

``` r
library(devtools)
install_github("scollinselliott/lakhesis", dependencies = TRUE, build_vignettes = TRUE) 
```

## Usage

To start the Lakhesis Calculator, execute the function `LC()`:

``` r
library(lakhesis)
LC()
```

In uploading a `csv` file for analysis inside the Lakhesis Calculator,
the incidence matrix should be in “long” format. That is, the file
should consist of just two columns without headers, in which each row
represents the incidence of a row-column pair. For example, an incidence
matrix of

$$\begin{array} \,  & C_1 & C_2 & C_3 \\\ R_1 & 1 & 0 & 0 \\\ R_2 &  0 & 1 & 1 \\\  R_3 & 0 & 0 & 1 \end{array}$$

will have a corresponding long format of

``` r
R1, C1
R2, C2 
R2, C3 
R3, C3
```

If characters are not displaying properly in the plot, make sure to
check font encoding (UTF-8 is recommended).

Row and column elements must be unique (a row element cannot have the
same name as a column element).

The Lakhesis Calculator enables the temporary suppression of row or
column elements from the plots, with zero rows/columns automatically
removed. As such, unexpected results may be elicited if key elements are
suppressed. All elements can easily be re-added and the starting
incidence matrix re-initialized.

### Incidence Matrices

If data are already in incidence matrix format, the `im_long()` function
in `lakhesis` can be used to convert an incidence matrix to be exported
into the necessary long format, using the `write.table()` function to
export (see documentation on `im_long()`):

``` r
# x is a matrix of 0/1 values with unique row/column names
y <- im_long(x)
write.table(y, file = "im.csv", sep = ",")
```

The file `im.csv` can then be loaded into the Lakhesis Calculator.

## Consensus Seriations

Establishing a consensus seriation via a lakhesis technique can be done
in the calculator, but if one has seriations, whether derived by
Procustes-fit CA or by another method, one can perform a consensus
seriation in the console by creating a `strands` object and then
executing the `lakhesize()` function.

The console can also be used to perform consensus seriations. For
example, using the built-in selection of three strands in the data
object `qf_strands`, a consensus seriation is performed using the
`lakhesize()` function:

``` r
x <- lakhesize(qf_strands)
summary(x)
```

The vignette “A Guide to Lakhesis” contains more information on usage.

## Citation

To cite `lakhesis`, please use the following reference:

- Collins-Elliott, S.A. 2026. “Lakhesis: Consensus Seriation via
  Iterative Regression of Partial Rankings for Binary Data.” *Journal of
  Applied Statistics*.
  [doi:10.1080/02664763.2026.2672564](https://doi.org/10.1080/02664763.2026.2672564).

## References

<div id="refs" class="references csl-bib-body hanging-indent"
entry-spacing="0">

<div id="ref-chang_shiny_2024" class="csl-entry">

Chang, W., J. Cheng, J. J. Allaire, C. Sievert, B. Schloerke, Y. Xie, J.
Allen, J. McPherson, A. Dipert, and B. Borges. 2024. *Shiny: Web
Application Framework for R*. <https://shiny.posit.co>.

</div>

<div id="ref-eddelbuettel_extending_2018" class="csl-entry">

Eddelbuettel, D., and J. J. Balamuta. 2018. “Extending R with C++: A
Brief Introduction to Rcpp.” *The American Statistician* 72: 28–36.
<https://doi.org/10.1080/00031305.2017.1375990>.

</div>

<div id="ref-eddelbuettel_rcpparmadillo_2014" class="csl-entry">

Eddelbuettel, D., and C. Sanderson. 2014. “RcppArmadillo: Accelerating R
<span class="nocase">with</span>
<span class="nocase">high</span>-<span class="nocase">performance</span>
C++ <span class="nocase">linear</span>
<span class="nocase">algebra</span>.” *Computational Statistics and Data
Analysis* 71: 1054–63. <https://doi.org/10.1016/j.csda.2013.02.005>.

</div>

<div id="ref-hahsler_getting_2008" class="csl-entry">

Hahsler, M., K. Hornik, and C. Buchcta. 2008. “Getting Things in Order:
An Introduction to the R Package Seriation.” *Journal of Statistical
Software* 25: 1–34. <https://doi.org/10.18637/jss.v025.i03>.

</div>

<div id="ref-ihm_contribution_2005" class="csl-entry">

Ihm, P. 2005. “A Contribution to the History of Seriation in
Archaeology.” In *Classification – The Ubiquitous Challenge*, edited by
C. Weihs and W. Gaul, 307–16. Berlin: Springer.

</div>

<div id="ref-oksanen_vegan_2024" class="csl-entry">

Oksanen, J., G. L. Simpson, F. G Blanchet, R. Kindt, P. Legendre, P. R.
Minchin, R. B. O’Hara, et al. 2024. “Vegan: Community Ecology Package.”
<https://doi.org/10.32614/CRAN.package.vegan>.

</div>

<div id="ref-ter_braak_weighted_1986" class="csl-entry">

ter Braak, C. J. F., and C. W. N. Looman. 1986. “Weighted Averaging,
Logistic Regression and the Gaussian Response Model.” *Vegetatio* 65:
3–11. <https://doi.org/10.1007/BF00032121>.

</div>

</div>

options(shiny.maxRequestSize=50*1024^2)

server <- function(input, output, session) { 

    # creating reactive values
    mats <- reactiveValues(mat = NULL, mat_initial = NULL, caproc = NULL, caproc_ref = NULL, nr = NULL, devr = NULL, devc = NULL)

    sup <- reactiveValues(rowsel = NULL, colsel = NULL)

    mats$mat <- im_ref(matrix(NA, 20, 20))
    mats$mat_initial <- im_ref(matrix(NA, 20, 20))
    mats$caproc <- ca_procrustes_ser(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
    mats$caproc_ref <- ca_procrustes(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
    mats$nr <- nrow(isolate(mats$caproc)$dat)
    mats$devr <- data.frame(Message = c("Lakhesize and run deviance test."))
    mats$devc <- data.frame(Message = c("Lakhesize and run deviance test."))
    sup$rowsel <- rownames(isolate(mats$mat_initial))
    sup$colsel <- colnames(isolate(mats$mat_initial))

    observeEvent(input$datafile, {
        mats$mat <- im_read_csv(input$datafile$datapath, char = input$char, remove.hapax = as.logical(input$hapax))
        mats$mat_initial <- mats$mat
        mats$caproc <- ca_procrustes_ser(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
        mats$caproc_ref <- ca_procrustes(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
        mats$nr <- nrow(isolate(mats$caproc)$dat)
        selections$caplot <- rep(FALSE, isolate(mats$nr))
        selections$biplot <- rep(FALSE, isolate(mats$nr))
        selections$curve <- rep(FALSE, isolate(mats$nr))

        sup$rowsel <- rownames(isolate(mats$mat_initial))
        sup$colsel <- colnames(isolate(mats$mat_initial))

        updateSelectInput(session,
            "suppressrows",
            choices = isolate(sup$rowsel)
        )
    
        updateSelectInput(session,
            "suppresscols",
            choices = isolate(sup$colsel)
        )
    })

    updateSelectInput(session,
        "suppressrows",
        choices = isolate(sup$rowsel)
    )
    
    updateSelectInput(session,
        "suppresscols",
        choices = isolate(sup$colsel)
    )

    # interactive plots
    selections <- reactiveValues(caplot = NULL, biplot = NULL, curve = NULL )
    selections$caplot <- rep(FALSE, isolate(mats$nr))
    selections$biplot <- rep(FALSE, isolate(mats$nr))
    selections$curve <- rep(FALSE, isolate(mats$nr))

    observeEvent(input$plot_brush_caplot, {
        brushed_caplot <- brushedPoints(mats$caproc$dat, input$plot_brush_caplot, allRows = TRUE)$selected_
        selections$caplot <- (brushed_caplot | selections$caplot)
    })

    observeEvent(input$plot_brush_biplot, {
        brushed_biplot <- brushedPoints(mats$caproc$dat, input$plot_brush_biplot, allRows = TRUE)$selected_
        selections$biplot <- (brushed_biplot | selections$biplot)
    })

    observeEvent(input$plot_brush_curve, {
        brushed_curve <- brushedPoints(mats$caproc$dat, input$plot_brush_curve, allRows = TRUE)$selected_
        selections$curve <- (brushed_curve | selections$curve)        
    })

    observeEvent(input$plot_reset, {
        nr <- mats$nr
        selections$caplot <- rep(FALSE, nr)
        selections$biplot <- rep(FALSE, nr)
        selections$curve <- rep(FALSE, nr)
    })

    output$caplot <- renderPlot({
        dat <- mats$caproc$dat
        dat$sel <- selections$caplot

        ca_plot <- ggplot2::ggplot() +
            ggplot2::geom_point(data = dat, ggplot2::aes(x = CA1, y = CA2, color = interaction(Type, sel, ":") ), size = 2) +
            ggplot2::geom_text(data = dat, ggplot2::aes(x = CA1, y = CA2, color = interaction(Type, sel, ":") ), label = rownames(dat), size = 3, hjust = 0.025, nudge_x = 0.015, check_overlap = TRUE) + 
            ggplot2::theme_bw() + ggplot2::theme(aspect.ratio = 1, legend.position="none")  
        ca_plot
    },  res = 96)

    output$biplot <- renderPlot({
        dat <- mats$caproc$dat
        dat$sel <- selections$biplot

        refcurve <- isolate(mats$caproc_ref)
        refcurve <- refcurve$ref
        refcurve <- data.frame(Procrustes1 = refcurve[,1], Procrustes2 = refcurve[,2])

        curve.plot <- ggplot2::ggplot() +
            ggplot2::geom_point(data = dat, ggplot2::aes(x = Procrustes1, y = Procrustes2, color = interaction(Type, sel, ":") ), size = 2) +
                ggplot2::geom_text(data = dat, ggplot2::aes(x = Procrustes1, y = Procrustes2, color = interaction(Type, sel, ":") ), label = rownames(dat), size = 3, hjust = 0.025, nudge_x = 0.015, check_overlap = TRUE) + 
                ggplot2::geom_line(data = refcurve,  ggplot2::aes(x = Procrustes1, y = Procrustes2), linewidth=1, alpha=0.4, linetype=1) + 
                ggplot2::theme_bw() + ggplot2::theme(aspect.ratio = 1, legend.position="none")  
        curve.plot
    },  res = 96)

    output$procrustesplot <- renderPlot({
        dat <- mats$caproc$dat
        dat$sel <- selections$curve #selected_curve()

        ord.plot <- ggplot2::ggplot() + 
            ggplot2::geom_point(data = dat, ggplot2::aes(x = CurveIndex, y = Distance, color = interaction(Type, sel, ":") ), size = 2) + 
                ggplot2::geom_text(data = dat, ggplot2::aes(x = CurveIndex, y = Distance, color = interaction(Type, sel, ":") ), label = rownames(dat), hjust = "left", nudge_y = 0.0005, size = 3,  angle = 90,  check_overlap = TRUE) + 
                ggplot2::theme_bw() +  ggplot2::theme(aspect.ratio = .3, legend.position="none")
        ord.plot
    }, res = 96)

    # re-run CA-Procrustes on selected points
    observeEvent(input$rerun, {
        m <- mats$mat
        keepRow <- !(rownames(m) %in% input$removeRow)
        keepCol <- !(colnames(m) %in% input$removeCol)
        keep.rc <- c(keepRow, keepCol)
        keep <- ((selections$caplot | selections$biplot)  | selections$curve ) & keep.rc
        dat <- mats$caproc$dat

        if ((length(rownames(dat)[keep & (dat$Type == "row") ]) > 2) & (length(rownames(dat)[keep & (dat$Type == "col") ]) > 2)) {

            tmp <- mats$mat[rownames(dat)[keep & (dat$Type == "row") ], rownames(dat)[keep & (dat$Type == "col") ] ]  
            tmp <- tmp[rowSums(tmp) != 0 , ]
            tmp <- tmp[,  colSums(tmp) != 0]

            if ( all( is.finite( tmp ) ) ) {

            mats$mat <- tmp
            mats$caproc <- ca_procrustes_ser(mats$mat, symmetric = isolate(as.logical(input$sym)))
            mats$caproc_ref <- ca_procrustes(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
            mats$nr <- nrow(mats$caproc$dat)

            selections$caplot <- rep(FALSE, isolate(mats$nr))
            selections$biplot <- rep(FALSE, isolate(mats$nr))
            selections$curve <- rep(FALSE, isolate(mats$nr))
            } else {
                message("Infinite values in selection.")
            }
        } else {
            message("Insufficient number of points selected.")
        }
    })

    # re-run CA on selected points
    observeEvent(input$rerun, {
        m <- mats$mat
        keepRow <- !(rownames(m) %in% input$removeRow)
        keepCol <- !(colnames(m) %in% input$removeCol)
        keep.rc <- c(keepRow, keepCol)
        keep <- ((selections$caplot | selections$biplot)  | selections$curve ) & keep.rc
        dat <- mats$caproc$dat

        if ((length(rownames(dat)[keep & (dat$Type == "row") ]) > 2) & (length(rownames(dat)[keep & (dat$Type == "col") ]) > 2)) {

            tmp <- mats$mat[rownames(dat)[keep & (dat$Type == "row") ], rownames(dat)[keep & (dat$Type == "col") ] ]  
            tmp <- tmp[rowSums(tmp) != 0 , ]
            tmp <- tmp[,  colSums(tmp) != 0]

            if ( all( is.finite( tmp ) ) ) {

            mats$mat <- tmp
            mats$caproc <- ca_procrustes_ser(mats$mat, symmetric = isolate(as.logical(input$sym)))
            mats$caproc_ref <- ca_procrustes(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
            mats$nr <- nrow(mats$caproc$dat)

            selections$caplot <- rep(FALSE, isolate(mats$nr))
            selections$biplot <- rep(FALSE, isolate(mats$nr))
            selections$curve <- rep(FALSE, isolate(mats$nr))
            } else {
                message("Infinite values in selection.")
            }
        } else {
            message("Insufficient number of points selected.")
        }
    })

    # restart the Lakhesis Calculator with the starting matrix
    observeEvent(input$reinitialize, {
        selections$caplot[] <- FALSE
        selections$biplot[] <- FALSE
        selections$curve[] <- FALSE

        mats$mat <- mats$mat_initial
        mats$caproc <- ca_procrustes_ser(mats$mat, symmetric = isolate(as.logical(input$sym)))
        mats$caproc_ref <- ca_procrustes(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
        mats$nr <- nrow(mats$caproc$dat)
        selections$caplot <- rep(FALSE, isolate(mats$nr))
        selections$biplot <- rep(FALSE, isolate(mats$nr))
        selections$curve <- rep(FALSE, isolate(mats$nr))
    })

    # processing results

    # reactives
    results <- reactiveValues(strands = list(), strand_hold = list,  strand_backup = NULL, lakhesis_results = NULL, lakhesized = FALSE)

    # record the observed seriation as a strand
    observeEvent(input$log_ca1, {
        mats$caproc <- ca_procrustes_ser(mats$mat, projection = "ca1", symmetric = isolate(as.logical(input$sym)))
        results$strands <- strand_add(isolate(mats$caproc), results$strands)
        results$strand_backup <- isolate(results$strands)
        #print(isolate(results$strands))
    })
    observeEvent(input$log_ca2, {
        mats$caproc <- ca_procrustes_ser(mats$mat, projection = "ca2", symmetric = isolate(as.logical(input$sym)))
        results$strands <- strand_add(isolate(mats$caproc), results$strands)
        results$strand_backup <- isolate(results$strands)
        #print(isolate(results$strands))
    })
    observeEvent(input$log_proc1, {
        mats$caproc <- ca_procrustes_ser(mats$mat, projection = "procrustes1", symmetric = isolate(as.logical(input$sym)))
        results$strands <- strand_add(isolate(mats$caproc), results$strands)
        results$strand_backup <- isolate(results$strands)
        #print(isolate(results$strands))
    })
    observeEvent(input$log_proc2, {
        mats$caproc <- ca_procrustes_ser(mats$mat, projection = "procrustes2", symmetric = isolate(as.logical(input$sym)))
        results$strands <- strand_add(isolate(mats$caproc), results$strands)
        results$strand_backup <- isolate(results$strands)
        #print(isolate(results$strands))
    })
    observeEvent(input$log_curve, {
        mats$caproc <- ca_procrustes_ser(mats$mat, projection = "curve", symmetric = isolate(as.logical(input$sym)))
        results$strands <- strand_add(isolate(mats$caproc), results$strands)
        results$strand_backup <- isolate(results$strands)
        #print(isolate(results$strands))
    })

    # lachesize strands
    observeEvent(input$lakhesize, {    
        if (length(isolate(results$strands)) > 1) {
            s <- isolate(results$strands)
            results$strand_backup <- isolate(results$strands)

            results$lakhesis_results <- lakhesize(s, crit = input$crits, pbar = FALSE) 

            if ("lakhesis" %in% class(isolate(results$lakhesis_results))) {
                results$lakhesized <- TRUE
            }

            selections$caplot[] <- FALSE
            selections$biplot[] <- FALSE
            selections$curve[] <- FALSE

            mats$mat <- mats$mat_initial
            mats$caproc <- ca_procrustes_ser(mats$mat, symmetric = isolate(as.logical(input$sym)))
            mats$caproc_ref <- ca_procrustes(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
            mats$nr <- nrow(mats$caproc$dat)

            selections$caplot <- rep(FALSE, mats$nr)
            selections$biplot <- rep(FALSE, mats$nr)
            selections$curve <- rep(FALSE, mats$nr)
            
        }
    })

    # # plots output after lakhesize() is run
    # output$consensusrowplot <- renderPlot({
    #     if (results$lakhesized == TRUE) {
    #         L <- results$lakhesis_results
    #         plot(L, display = "rowPCA")
    #     } else {
    #         plot(0,0, pch = " ", xlab = "Lakhesize to produce consensus PCA biplot.", ylab = " " )
    #     }
    # })

    # output$consensuscolplot <- renderPlot({
    #     if (results$lakhesized == TRUE) {
    #         L <- results$lakhesis_results
    #         plot(L, display = "colPCA")
    #     } else {
    #         plot(0,0, pch = " ", xlab = "Lakhesize to produce consensus PCA biplot.", ylab = " " )
    #     }
    # })

    output$consensusmatrixplot <- renderPlot({
        if (results$lakhesized == TRUE) {
            L <- results$lakhesis_results
            plot(L, display = "im_seriated")

        } else {
            plot(0,0, pch = " ", xlab = "Lakhesize to produce consensus matrix plot.", ylab = " " )
        }
    })

    # coefficient plots
    output$consensusplot <- renderPlot({
        if (results$lakhesized == TRUE) {
            L <- results$lakhesis_results
            #lakhcoef$Strand <- factor(lakhcoef$Strand, levels = lakhcoef$Strand[order(lakhcoef$Agreement, decreasing = FALSE)]) 
            suppressWarnings({
                plot(L, display = "agreement")
            })
            } else {
                plot(0,0, pch = " ", xlab = "Lakhesize to render plot of coefficents for strands.", ylab = " " )
        } 
    })

    output$kappaplot <- renderPlot({
        if (results$lakhesized == TRUE) {
            L <- results$lakhesis_results
            suppressWarnings({
                plot(L, display = "criterion")
            })
            } else {
                plot(0,0, pch = " ", xlab = "Lakhesize to render plot of coefficents for strands.", ylab = " " )
        }   
    })

    # delete a strand
    observeEvent(input$submitStrandDelete, {
        if (results$lakhesized == TRUE) {

            delValue <- as.integer(input$strandDelete)
            
            results$strand_hold <- isolate(results$strands[delValue])

            results$strands[delValue] <- NULL

            if (length(results$strands) > 1) {

            mats$mat <- mats$mat_initial
            suppressWarnings({
               results$lakhesis_results <- lakhesize(isolate(results$strands), crit = input$crits, pbar = FALSE) 
            })
            selections$caplot[] <- FALSE
            selections$biplot[] <- FALSE
            selections$curve[] <- FALSE

            mats$mat <- mats$mat_initial
            mats$caproc <- ca_procrustes_ser(mats$mat, symmetric = isolate(as.logical(input$sym)))
            mats$caproc_ref <- ca_procrustes(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
            mats$nr <- nrow(mats$caproc$dat)
            } else {
                mats$mat <- mats$mat_initial
                mats$caproc <- ca_procrustes_ser(mats$mat, symmetric = isolate(as.logical(input$sym)))
                mats$caproc_ref <- ca_procrustes(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
                mats$nr <- nrow(mats$caproc$dat)
            }
            selections$caplot[] <- FALSE
            selections$biplot <- rep(FALSE, mats$nr)
            selections$curve <- rep(FALSE, mats$nr)
        }
    })

    observeEvent(input$submitStrandDeleteUndo, {
        if (results$lakhesized == TRUE) {

            results$strands[[length(results$strands) + 1]] <- isolate(results$strand_hold[[1]])

            results$strand_hold <- NULL

            mats$mat <- mats$mat_initial
            suppressWarnings({
            results$lakhesis_results <- lakhesize(isolate(results$strands), crit = input$crits, pbar = FALSE) 
            })
            selections$caplot[] <- FALSE
            selections$biplot[] <- FALSE
            selections$curve[] <- FALSE

            mats$mat <- mats$mat_initial
            mats$caproc <- ca_procrustes_ser(mats$mat, symmetric = isolate(as.logical(input$sym)))
            mats$caproc_ref <- ca_procrustes(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
            mats$nr <- nrow(mats$caproc$dat)

            selections$caplot[] <- FALSE
            selections$biplot <- rep(FALSE, mats$nr)
            selections$curve <- rep(FALSE, mats$nr)
        }
    })

    # deviance test results
    observeEvent(input$deviancetest, {
        if (results$lakhesized == TRUE) {
            L <- isolate(results$lakhesis_results)

            dev <- element_eval(L$im_seriated)
            mats$devc <- head(dev$Col, n = 10L)
            mats$devr <- head(dev$Row, n = 10L)
        }
    })

    output$deviancerows <- renderTable(mats$devr)
    output$deviancecols <- renderTable(mats$devc)

    # export data to a .rds object
    output$downloadData <- downloadHandler(
        filename = function() {
            paste("Lakhesis", format(Sys.Date(), '%Y%m%d'), format(Sys.time(), '%H%M%S'), ".rds", sep="")
        },
        content = function(file) {
            # reset plot when exporting
            results$lakhesized <- TRUE

            selections$caplot[] <- FALSE
            selections$biplot[] <- FALSE
            selections$curve[] <- FALSE

            # lakhesize to ensure that all row/col elements from strands are in results if user has not performed this action
            s <- isolate(results$strands)
            suppressWarnings({
            results$lakhesis_results <- lakhesize(s, crit = input$crits, pbar = FALSE) 
            })
            lr <- isolate(results$lakhesis_results)
            results <- list(consensus = lr, strands = s)
            saveRDS(results, file = file)
          
            mats$mat <- mats$mat_initial
            mats$caproc <- ca_procrustes_ser(mats$mat, symmetric = isolate(as.logical(input$sym)))
            mats$caproc_ref <- ca_procrustes(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
            mats$nr <- nrow(mats$caproc$dat)

            selections$caplot[] <- FALSE
            selections$biplot <- rep(FALSE, mats$nr)
            selections$curve <- rep(FALSE, mats$nr)
        }
    )

    observeEvent(input$suppressrows, {
        if (!is.null(input$suppressrows)) {
            mi <- isolate(mats$mat_initial)
            sr <- isolate(input$suppressrows)
            mi <- mi[(!(rownames(mi) %in% sr)), ]
            mi <- mi[ , (colSums(mi) != 0)]
            mats$mat <- mi
            mats$caproc <- ca_procrustes_ser(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
            mats$caproc_ref <- ca_procrustes(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
            mats$nr <- nrow(isolate(mats$caproc$dat))
            selections$caplot <- rep(FALSE, isolate(mats$nr))
            selections$biplot <- rep(FALSE, isolate(mats$nr))
            selections$curve <- rep(FALSE, isolate(mats$nr))
        }
    })

    observeEvent(input$suppresscols, {
        if (!is.null(input$suppresscols)) {
            mi <- isolate(mats$mat_initial)
            sc <- isolate(input$suppresscols)
            mi <- mi[, (!(colnames(mi) %in% sc)) ]
            mi <- mi[(rowSums(mi) != 0), ]
            mats$mat <- mi
            mats$caproc <- ca_procrustes_ser(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
            mats$caproc_ref <- ca_procrustes(isolate(mats$mat), symmetric = isolate(as.logical(input$sym)))
            mats$nr <- nrow(isolate(mats$caproc$dat))
            selections$caplot <- rep(FALSE, isolate(mats$nr))
            selections$biplot <- rep(FALSE, isolate(mats$nr))
            selections$curve <- rep(FALSE, isolate(mats$nr))
        }
    })

    observeEvent(input$end, {
            session$close()
            stopApp()
    })

}

ui <- shinydashboard::dashboardPage(
  shinydashboard::dashboardHeader(title = "Lakhesis Calculator"),  
  shinydashboard::dashboardSidebar(
        shiny::column(12,align = "center",offset = 0,
        shiny::radioButtons("hapax", label = "Selection:", choices = c("Use All Data" = FALSE, "Remove Hapax" = TRUE)),
        shiny::radioButtons("char", label = "Encoding:", choices = c("ISO-8859-1" = "iso-8859-1", "UTF-8" = "utf-8")),
        shiny::fileInput('datafile', 'Choose CSV File:',
                accept=c('csv', 'comma-separated-values','.csv')),
        shiny::radioButtons("sym", label = "CA Scores:", choices = c("Symmetric" = TRUE, "Asymmetric" = FALSE)),
        shiny::actionButton("reinitialize", "Reinitialize", width = "90%"),
        shiny::actionButton("rerun", "Replot with Selection", width = "90%"),
        # shiny::actionButton("log", "Save Seriation as Strand", width = "90%"),

        shiny::br(),
        shiny::downloadButton("downloadData", "Export Data", width = "90%"),
        shiny::br(),shiny::br(),
        shiny::tags$img(src = file.path('img', 'logo.png'), height = "50%", width = "50%", align="center"),
        shiny::br(),shiny::br(),
        shiny::actionButton("end", "End Calculator", width = "90%")
        )
        ),
    shinydashboard::dashboardBody(  
        shiny::fluidRow(
            shinydashboard::tabBox(title = "Seriation Explorer",
                id = "pointselector", height = "500px", width = 5, 
                    shiny::tabPanel("CA Plot",
                    shiny::plotOutput(outputId="caplot", brush = shiny::brushOpts(id="plot_brush_caplot", resetOnNew = TRUE), dblclick = "plot_reset"),
                    #column(12,align = "center",offset = 0,
                    #shiny::actionButton("save_curve", "Gather Plot Points", width = "90%"))
                ),
                shiny::tabPanel("CA-Procrustes Plot",
                    shiny::plotOutput(outputId="biplot", brush = shiny::brushOpts(id="plot_brush_biplot", resetOnNew = TRUE), dblclick = "plot_reset"),
                    #column(12,align = "center",offset = 0,
                    #shiny::actionButton("save_biplot", "Gather Plot Points", width = "90%"))
                ),
                shiny::tabPanel("Curve Plot",
                    shiny::plotOutput(outputId="procrustesplot", brush = shiny::brushOpts(id="plot_brush_curve", resetOnNew = TRUE), dblclick = "plot_reset"),
                    #column(12,align = "center",offset = 0,
                    #shiny::actionButton("save_curve", "Gather Plot Points", width = "90%"))
                )
            ),
            shinydashboard::tabBox(title = "Save Strand",
                id = "pcaplot", height = "500px", width = 2, 
                shiny::tabPanel("Projection",
                shiny::column(12,align = "center",offset = 0,
                p("Select a strand by choosing which axis to use for its seriation."),
                shiny::actionButton("log_ca1", "CA1", width = "90%"),
                shiny::actionButton("log_ca2", "CA2", width = "90%"),
                shiny::actionButton("log_proc1", "Procrustes1", width = "90%"),
                shiny::actionButton("log_proc2", "Procrustes2", width = "90%"),
                shiny::actionButton("log_curve", "Curve", width = "90%"),
                shiny::br(),
                shiny::radioButtons("crits", label = "Criterion:", choices = c("cor_sq" = "cor_sq", "conc_wrc" = "conc_wrc")),
                shiny::actionButton("lakhesize", "Lakhesize (Consensus)", width = "90%"),
                shiny::actionButton("deviancetest", "Run Deviance Test", width = "90%")

                )
                )
            ),
            shinydashboard::tabBox(title = "Consensus Seriation",
                id = "pcaplot", height = "500px", width = 5, 
                shiny::tabPanel("Matrix",
                    shiny::plotOutput(outputId="consensusmatrixplot")
                )
            )
        ),
        shiny::fluidRow(
            shinydashboard::tabBox(title = "Diagnostics",
                id = "results",
                # shiny::tabPanel("Lakhesis Coefficient",   
                #     shiny::plotOutput(outputId="lakhesiscoefplot")
                # ),
                shiny::tabPanel("Agreement",   
                    shiny::plotOutput(outputId="consensusplot")
                ),
                shiny::tabPanel("Criterion",   
                    shiny::plotOutput(outputId="kappaplot")
                ),
                shiny::tabPanel("Deviance (Rows)",   
                    shiny::tableOutput("deviancerows")
                ),
                shiny::tabPanel("Deviance (Columns)",
                    shiny::tableOutput("deviancecols")
                )
            ),
            shinydashboard::box(title = "Modify", 
                selectInput("suppressrows", label = "Select Rows to Suppress", choices = "rowsel", multiple = TRUE),
                selectInput("suppresscols", label = "Select Columns to Suppress", choices = "colsel", multiple = TRUE),
                shiny::p("Currently suppressed row or column values will not be included when saving strands, and are not removed from previously saved strands."),
                shiny::br(),
                shiny::textInput("strandDelete", "Delete Strand (Indices Will Resort Automatically, Undo Will Re-Add as Last Strand)"),
                shiny::actionButton("submitStrandDelete", "Delete"),
                shiny::actionButton("submitStrandDeleteUndo", "Undo")
                )
        )
    )
)

#' Read csv File to Incidence Matrix
#'
#' Wrapper around the \code{\link[readr]{read_csv}} function from the \code{readr} package \insertCite{wickham_readr_2024}{lakhesis}. Read a \code{.csv} file in which the first column represents row elements and the second column represents column elements, and convert it into an incidence matrix.
#'  
#' @param filename The filename to uploaded (must be in \code{.csv} format).
#' @param header If the \code{.csv} file contains a header. Default is `FALSE`.
#' @param characterencoding File encoding as used by \code{\link[readr]{locale}}, which depends on the \code{csv} file. Default is \code{"iso-8859-1"} but \code{"utf-8"} is also recommended. 
#' @param remove.hapax Remove any row or column which has a sum of 1 (i.e., is only attested once), since they do not directly contribute to the result of the seriation. Default is `FALSE`.
#' @returns A matrix of binary values (0 = row/column occurrence is absence; 1 = row/column occurrence is present). 
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' @importFrom Rdpack reprompt
im_read_csv <- function(filename, header = FALSE, characterencoding = "iso-8859-1", remove.hapax = FALSE) {
    dat <- readr::read_csv(filename, col_names = header, show_col_types = FALSE, locale = readr::locale(encoding = characterencoding))
    dat <- data.frame(dat)

    finds <- levels(factor(dat[,2]))
    contexts <- levels(factor(dat[,1]))
    obj <- matrix(0, nrow = length(contexts), ncol = length(finds))
    rownames(obj) <- contexts
    colnames(obj) <- finds

    for (k in 1:nrow(dat)) {
        j <- which(finds == dat[k, 2]) 
        i <- which(contexts == dat[k, 1])
        obj[i, j] <- 1
    }

    if (remove.hapax == TRUE) {
        obj <- obj[, colSums(obj) > 1]
        obj <- obj[rowSums(obj) > 1, ]
    }

    class(obj) <- c("incidence_matrix", "matrix", "array")
    return(obj)
}



#' Create Reference Matrix
#'
#' Create an ideal reference matrix of well-seriated values of the same size as the input matrix. 
#'
#' @param obj A matrix of size \eqn{n \times k}.
#' @returns A matrix of size \eqn{n \times k} with 1s along the diagonal. If \eqn{n > k}, 1s are placed from cell \eqn{(i,i)} to \eqn{(i,i+k-n)}, with 0 in all other cells.
#' @examples
#' im_ref(matrix(NA, 5, 5))
#' im_ref(matrix(1, 7, 12))
#' 
#' @export
im_ref <- function(obj) {
    UseMethod("im_ref")
}

#' @rdname im_ref
#' @export
im_ref.matrix <- function(obj) { 
        ref <- matrix(0, nrow = nrow(obj), ncol = ncol(obj))
        if (nrow(ref) > ncol(ref)) {
            r <- nrow(ref) - ncol(ref) 
            for (j in 1:ncol(ref)) {
                end <- j+r
                ref[j:end,j] <- 1
            } 
        } else if (nrow(ref) == ncol(ref)) {
            diag(ref) <- 1
            r <- nrow(ref) - ncol(ref) 
            for (j in 1:(ncol(ref)-1)) {
                end <- j+1
                ref[j:end,j] <- 1
            } 
        } else {
            r <- ncol(ref) - nrow(ref) 
            for (i in 1:nrow(ref)) {
                end <- i+r
                ref[i, i:end] <- 1
            }
        }
        if (is.null(colnames(ref))) {
            rownames(ref) <- paste("R", sprintf("%02.0f", 1:nrow(ref)), sep = '')
            colnames(ref) <- paste("C", sprintf("%02.0f", 1:ncol(ref)), sep = '')
        }
    class(ref) <- c("incidence_matrix", "matrix", "array")
    return(ref)
}


#' @export
im_ref.incidence_matrix <- function(obj) { 
    im_ref.matrix(obj)
}


#' Convert Incidence Matrix to Pairs (Long Format)
#' 
#' Take an incidence matrix and convert it to a data frame of two columns, where the first column represents the row elements of the incidence matrix and the second column represents the column elements of the incidence matrix. Each row pair represents the incidence (or occurrence) of that row and column element together.
#'
#' @param obj An incidence matrix. 
#' @returns A data frame of two columns (row and column of the incidence matrix), in which row of the data frame represents a pair of an 
#' @examples 
#' data(quattrofontanili)
#' qf <- im_long(quattrofontanili)
#' 
#' # to export for uploading into the Lakhesis Calculator, use write.table() to 
#' # remove both row and column names:
#' 
#' # write.table(qf, file = 'qf.csv', row.names = FALSE, col.names = FALSE, sep = ",")
#' 
#' @export
im_long <- function(obj) {
    UseMethod("im_long")
}

#' @rdname im_long
#' @export
im_long.matrix <- function(obj) { 
    dat <- data.frame(Row = c(), Col = c())
    for (i in 1:nrow(obj)) {
        row <- obj[i, ]
        rows <- rownames(obj)[i]
        cols <- names(row[row == 1])
        tmp <- matrix(c(rep(rows, length(cols)) , cols), nrow = length(cols), ncol = 2)
        dat <- rbind(dat, tmp)
    }
    colnames(dat) <- c("Row", "Col")
    class(dat) <- "data.frame"
    return(dat)
}

#' @rdname im_long
#' @export
im_long.incidence_matrix <- function(obj) { 
    im_long.matrix(obj)
}

#' Merge Two Incidence Matrices
#'
#' From two incidience matrices, create a single incidence matrix. Matrices may contain same row or column elements. 
#'
#' @param obj1,obj2 Two incidence matrices of any size.
#' @returns A single incidence matrix.
#' @examples 
#' data(quattrofontanili)
#' qf1 <- quattrofontanili[1:20, 1:40]
#' qf1 <- qf1[rowSums(qf1) != 0, colSums(qf1) != 0]
#' 
#' qf2 <- quattrofontanili[30:50, 20:60]
#' qf2 <- qf2[rowSums(qf2) != 0, colSums(qf2) != 0]
#' 
#' im_merge(qf1, qf2)
#' 
#' @export
im_merge <- function(obj1, obj2) {
    UseMethod("im_merge")
}

#' @rdname im_merge
#' @export
im_merge.matrix <- function(obj1, obj2) {
    dat <- rbind(im_long(obj1), im_long(obj2))

    finds <- levels(factor(dat[,2]))
    contexts <- levels(factor(dat[,1]))
    obj <- matrix(0, nrow = length(contexts), ncol = length(finds))
    rownames(obj) <- contexts
    colnames(obj) <- finds

    for (k in 1:nrow(dat)) {
        j <- which(finds == dat[k, 2]) 
        i <- which(contexts == dat[k, 1])
        obj[i, j] <- 1
    }
    class(obj) <- c("incidence_matrix", "matrix", "array")
    return(obj)
}

#' @rdname im_merge
#' @export
im_merge.incidence_matrix <- function(obj1, obj2) {
    im_merge.matrix(obj1, obj2)
}

#' Lakhesize
#'
#' This function returns the row and column consensus seriation for a \code{list} object of the \code{strands} class, containing their rankings, coefficients of association, and criterion. Consensus seriation is achieved by iterative simple linear regression to handle \code{NA} vales in each strand. To initialize, a regression is performed pairwise, with every strand as the dependent \eqn{y} variate and every other strand as the independent \eqn{x} variate. The independent variate's rankings are then regressed onto \eqn{f(x) = \hat{\beta}_1 x + \hat{\beta}_0}. If \eqn{y \neq f(x)}, the mean of \eqn{y} and \eqn{f(x)} is used. Then, the values of the dependent variate and those of the regressed independent varaite are re-ranked together, to form a combined ranking, which serves as the dependent variate on the next iteration. The pair of strands is chosen which minimizes a specified optimality criterion. The process is repeated until all strands have been regressed and re-ranked into a single consensus seriation.
#'
#' @param strands A \code{list} of \code{strands} class (see \code{\link[lakhesis]{strand_add}}).
#' @param crit The criterion used to assess the seration resulting from two strands \eqn{x} and \eqn{y}: 
#'   * \code{"cor_sq"} Computes a squared correlation coefficent for the incidences of 1s as points \eqn{(i,j)}. See \code{\link[lakhesis]{cor_sq}}. Higher values are more optimal. This is the default option.
#'   * \code{"conc_wrc"} Computes weighted row-column concentration as the optimality criterion. See \code{\link[lakhesis]{conc_wrc}}. Lower values are more optimal.
#' 
#' @param pbar Displaying a progress bar. Default is \code{TRUE}.
#' @return A \code{list} of class \code{lakhesis} containing the following:
#' 
#' * \code{row} A seriated vector of row elements. 
#' * \code{col} A seriated vector of column elements
#' * \code{coef}  A \code{data frame} containing the following columns:
#'   * \code{Strand} The number of the strand.
#'   * \code{Agreement} The measure of agreement, i.e., how well each strand accords with the consensus seriation. Using the square of Spearman's rank correlation coefficient, \eqn{\rho^2}, between each strand and the consensus ranking, agreement is computed as the product of \eqn{\rho^2} for their row and column rankings, \eqn{\rho_r^2}\eqn{\rho_c^2}. 
#'   * \code{Criterion} Criterion of the optimality of each strand (per the \code{"crit"} option above).
#' * \code{im_seriated} The seriated incidence matrix, of class \code{incidence_matrix}.
#' 
#' @examples
#' data("qf_strands")
#' L <- lakhesize(qf_strands, pbar = FALSE)
#' # summary(L) 
#'
#' @export
lakhesize <- function(strands, crit = "cor_sq", pbar = TRUE) {
    UseMethod("lakhesize")
}

#' @rdname lakhesize
#' @export 
lakhesize.strands <- function(strands, crit = "cor_sq", pbar = TRUE) {
    lakhesize.default(strands, crit, pbar)
}

#' @rdname lakhesize
#' @export
lakhesize.default <- function(strands, crit = "cor_sq", pbar = TRUE) {    
    if (!(crit %in% c("cor_sq", "conc_wrc"))) {
        stop('Criterion must be "cor_sq" or "conc_wrc".')
    }
    obj <- strands[[1]]$im_seriated
    for (i in 2:length(strands)) {
        obj <- im_merge(obj, strands[[i]]$im_seriated)
    }
    ns <- length(strands)

    if (ns > 1) {
        strand.mat <- strand_extract(strands)

        rowranks <- strand.mat[["Row"]]
        colranks <- strand.mat[["Col"]]

        check_matrix <- matrix(NA, ns, ns)
        for (i in 1:ns) {
            for (j in i:ns) {
                if (i != j) {
                    dat <- rowranks[,c(i,j)]
                    check_matrix[i,j] <- nrow( dat[!is.na(rowSums(dat)),] ) 
                }
            }
        }

        check_matrix <- check_matrix > 3

        if (ns > 2) {
            check <- colSums(check_matrix[ ,2:ncol(check_matrix)], na.rm = TRUE) + rowSums(check_matrix[1:(ncol(check_matrix)-1) ,], na.rm = TRUE) 
        }

        if (ns == 2) {
            check <- check_matrix[1,2]
        }

        if (0 %in% check ) {
            warning("Each strand must share at least four joint elements with another strand.")
            return(NULL)
        } else {
            if (pbar == TRUE) {
                pb <- utils::txtProgressBar(min = 0, max = ns, style = 3)
            }   
            regressed <- c()
            criterion_matrix <- matrix(NA, ns, ns)

            for (i in 1:ns) {
                for (j in i:ns) {
                    if (!(i == j)) {
                        rdat <- data.frame(rowranks[,i], rowranks[,j])
                        rdat1 <- rdat[!is.na(rowSums(rdat)),]
                        cdat <- data.frame(colranks[,i], colranks[,j])
                        cdat1 <- cdat[!is.na(rowSums(cdat)),]
                        if ((nrow(rdat1) > 3) & (nrow(cdat1) > 3) ) {
                            ry <- rdat1[,2]
                            rx <- rdat1[,1]
                            rfit <- stats::lm(ry ~ rx)

                            rdat[,1] <- rdat[,1] * rfit$coef[2] + rfit$coef[1]

                            rdat <- rowMeans(rdat, na.rm = TRUE)
                            rdat <- rank(rdat, na.last = "keep")

                            cy <- cdat1[,2]
                            cx <- cdat1[,1]
                            cfit <- stats::lm(cy ~ cx)

                            cdat[,1] <- cdat[,1] * cfit$coef[2] + cfit$coef[1]

                            cdat <- rowMeans(cdat, na.rm = TRUE)
                            cdat <- rank(cdat, na.last = "keep")

                            R <- names(rdat)[order(rdat)]
                            C <- names(cdat)[order(cdat)]
                            if (crit == "conc_wrc") {
                                K <- conc_wrc(obj[R, C])
                            } else if (crit == "cor_sq") {
                                K <- cor_sq(obj[R, C])
                            }
                            criterion_matrix[i,j] <- K
                        }
                    }
                }
            }

            regressed <- rev( arrayInd(which.min(criterion_matrix), dim(criterion_matrix)))
            remaining <- 1:ns
            remaining <- remaining[-regressed]

            rdat <- data.frame(rowranks[,regressed[2]], rowranks[,regressed[1]])
            rdat1 <- rdat[!is.na(rowSums(rdat)),]
            cdat <- data.frame(colranks[,regressed[2]], colranks[,regressed[1]])
            cdat1 <- cdat[!is.na(rowSums(cdat)),]

            ry <- rdat1[,2]
            rx <- rdat1[,1]
            rfit <- stats::lm(ry ~ rx)

            rdat[,1] <- rdat[,1] * rfit$coef[2] + rfit$coef[1]

            rdat <- rowMeans(rdat, na.rm = TRUE)
            rdat_y <- rank(rdat, na.last = "keep")

            cy <- cdat1[,2]
            cx <- cdat1[,1]
            cfit <- stats::lm(cy ~ cx)

            cdat[,1] <- cdat[,1] * cfit$coef[2] + cfit$coef[1]

            cdat <- rowMeans(cdat, na.rm = TRUE)
            cdat_y <- rank(cdat, na.last = "keep")

            while (length(remaining) > 0) {
                kappa_check = numeric(ns)
                kappa_check[] <- NA
                for (i in remaining) {
                    rdat <- data.frame(rowranks[,i], rdat_y)
                    rdat1 <- rdat[!is.na(rowSums(rdat)),]
                    cdat <- data.frame(colranks[,i], cdat_y)
                    cdat1 <- cdat[!is.na(rowSums(cdat)),]
                    if ((nrow(rdat1) > 3) & (nrow(cdat1) > 3) ) {
                        ry <- rdat1[,2]
                        rx <- rdat1[,1]
                        rfit <- stats::lm(ry ~ rx)

                        rdat[,1] <- rdat[,1] * rfit$coef[2] + rfit$coef[1]

                        rdat <- rowMeans(rdat, na.rm = TRUE)
                        rdat <- rank(rdat, na.last = "keep")

                        cy <- cdat1[,2]
                        cx <- cdat1[,1]
                        cfit <- stats::lm(cy ~ cx)

                        cdat[,1] <- cdat[,1] * cfit$coef[2] + cfit$coef[1]

                        cdat <- rowMeans(cdat, na.rm = TRUE)
                        cdat <- rank(cdat, na.last = "keep")

                        R <- names(rdat)[order(rdat)]
                        C <- names(cdat)[order(cdat)]

                        if (crit == "conc_wrc") {
                            K <- conc_wrc(obj[R, C])
                        } else if (crit == "cor_sq") {
                            K <- cor_sq(obj[R, C])
                        }

                        kappa_check[i] <- K
                    }
                }
                idx_next <- which.min(kappa_check)
                regressed <- c(regressed, idx_next)
                remaining <- remaining[-which(remaining == idx_next)]

                rdat <- data.frame(rowranks[,idx_next], rdat_y)
                rdat1 <- rdat[!is.na(rowSums(rdat)),]
                cdat <- data.frame(colranks[,idx_next], cdat_y)
                cdat1 <- cdat[!is.na(rowSums(cdat)),]

                ry <- rdat1[,2]
                rx <- rdat1[,1]
                rfit <- stats::lm(ry ~ rx)

                rdat[,1] <- rdat[,1] * rfit$coef[2] + rfit$coef[1]

                rdat <- rowMeans(rdat, na.rm = TRUE)
                rdat_y <- rank(rdat, na.last = "keep")

                cy <- cdat1[,2]
                cx <- cdat1[,1]
                cfit <- stats::lm(cy ~ cx)

                cdat[,1] <- cdat[,1] * cfit$coef[2] + cfit$coef[1]

                cdat <- rowMeans(cdat, na.rm = TRUE)
                cdat_y <- rank(cdat, na.last = "keep")

                if (pbar == TRUE) {
                    utils::setTxtProgressBar(pb, length(regressed))
                }   
            }

            RL <- names(rdat_y)[order(rdat_y)]
            CL <- names(cdat_y)[order(cdat_y)]

            if (pbar == TRUE) {
                close(pb)
            }

            # criterion of each strand
            strand.k.c <- c()
            if (crit == "conc_wrc") {
                for (i in 1:length(strands)) {
                    ctx <- stats::na.omit(rowranks[,i])
                    fnd <- stats::na.omit(colranks[,i])
                    roworder <- names(ctx)[order(ctx)]
                    colorder <- names(fnd)[order(fnd)]
                    strand.im <- obj[roworder,colorder]
                    k.c <- conc_wrc(strand.im)
                    strand.k.c <- c(strand.k.c, k.c)
                }
            } else if (crit == "cor_sq") {
                for (i in 1:length(strands)) {
                    ctx <- stats::na.omit(rowranks[,i])
                    fnd <- stats::na.omit(colranks[,i])
                    roworder <- names(ctx)[order(ctx)]
                    colorder <- names(fnd)[order(fnd)]
                    strand.im <- obj[roworder,colorder]
                    k.c <- cor_sq(strand.im)
                    strand.k.c <- c(strand.k.c, k.c)
                }
            }

            # agreement with conensus seration
            assoc <- c()
            for (i in 1:length(strands)) {
                sp.f <- spearman_sq( rowranks[,i], rdat_y )
                sp.c <- spearman_sq( colranks[,i], cdat_y )
                assoc <- c(assoc,  sp.f * sp.c)
            }

            coefs <- data.frame( Strand = 1:length(strands), Agreement = assoc, Criterion = strand.k.c)

            results <- list()

            results[["row"]] <- RL #consensus.row.dat
            results[["col"]] <- CL #consensus.col.dat
            results[["coef"]] <- coefs
            im <- obj[RL,CL]
            class(im) <- c("incidence_matrix", "matrix")
            results[["im_seriated"]] <- im
            class(results) <- c("lakhesis", "list")
            return(results)
        } 
    } else {
        warning("Need more than 1 strand to create consensus seration.")
    }
}

#' @export 
print.procrustean <- function(x, ...) {
        cat("Procrustes-fit CA scores of class", class(x), "\n")
        cat("  $ref : principal scores: reference points\n")
        cat("  $x : standard scores: row points\n")
        cat("  $y : principal scores: column points\n")
        cat("  $x_pr : Procrustes-fit standard scores: row points\n")
        cat("  $y_pr : Procrustes-fit principal scores: column points\n")
}



#' @export 
plot.procrustean <- function(x, bw = FALSE, ...) {
    Procrustes1 <- Procrustes2 <- scale_colour_grey <- NULL
    ref <- data.frame(Procrustes1 = x$ref[,1], Procrustes2 = x$ref[,2])
    r.dat <- data.frame(Procrustes1 = x$x_pr[,1], Procrustes2 = x$x_pr[,2])
    c.dat <- data.frame(Procrustes1 = x$y_pr[,1], Procrustes2 = x$y_pr[,2])
    Type <- c(rep("row", nrow(x$x)), rep("col", nrow(x$y)) )
    dat <- cbind( rbind(r.dat, c.dat), Type)
    #ref <- im_ref( matrix(NA, nrow = sum(strand$Type == 'row'),  ncol = sum(strand$Type == 'col')  ))
    #refcurve <- ca_procrustes_curve(ref)
    if (bw == FALSE) {
    curve.plot <- ggplot2::ggplot() +
        ggplot2::geom_point(data = dat, ggplot2::aes(x = Procrustes1, y = Procrustes2, color = Type), size = 2) +
        ggplot2::geom_text(data = dat, ggplot2::aes(x = Procrustes1, y = Procrustes2, color = Type ), label = rownames(dat), size = 3, hjust = 0.025, nudge_x = 0.015, check_overlap = TRUE) + 
        ggplot2::geom_line(data = ref,  ggplot2::aes(x = Procrustes1, y = Procrustes2), linewidth=1, alpha=0.4, linetype=1) +
        ggplot2::theme_bw() + ggplot2::theme(aspect.ratio = 1, legend.position="none") 
    } else if (bw == TRUE) {
    curve.plot <- ggplot2::ggplot() +
        ggplot2::geom_point(data = dat, ggplot2::aes(x = Procrustes1, y = Procrustes2, color = Type), size = 2) +
        ggplot2::geom_text(data = dat, ggplot2::aes(x = Procrustes1, y = Procrustes2, color = Type ), label = rownames(dat), size = 3, hjust = 0.025, nudge_x = 0.015, check_overlap = TRUE) + 
        ggplot2::geom_line(data = ref,  ggplot2::aes(x = Procrustes1, y = Procrustes2), linewidth=1, alpha=0.4, linetype=1) +
        scale_colour_grey(start = 0.8, end = 0) + 
        ggplot2::theme_bw() + ggplot2::theme(aspect.ratio = 1, legend.position="none")         
    }
    curve.plot
}



#' @export 
print.strand <- function(x, ...) {
    cat("Procrustes-fit CA of class:", class(x), "\nUse summary() for more information. \n")
}



#' @export 
summary.strand <- function(object, ...) {
    s <- object$dat
    cat("Procrustes-fit CA of class:", class(object), "\n")
    cat("\n")
    cat("Ranking of row elements: \n")
    rows <- s[s$Type == "row", ]
    cat(rownames(rows)[order(rows$Rank)], fill = TRUE)
    cat("\n")
    cat("Ranking of column elements: \n")
    cols <- s[s$Type == "col", ]
    cat(rownames(cols)[order(cols$Rank)], fill = TRUE)
    cat("\n")
    cat("$dat contains the following columns:\n")
    cat("   $Procrustes1, $Procrustes2 : x,y coords for the Procrustes-fit CA principal scores\n")
    cat("   $CurveIndex: index of nearest point on refrence curve to the Procrustes-fit CA score point\n")
    cat("   $Distance: distance of principal score point to nearest point on reference curve\n")
    cat("   $Rank: ranking of score points projected onto the reference curve\n")
    cat('   $Type: "row" or "col"\n')
    cat("$im_seriated: the seriated incidence matrix")
    cat('\n')
}



#' @export 
plot.strand <- function(x, display = "ca", ...) {
    CurveIndex <- Distance <- Type <- NULL
    #ref <- im_ref( matrix(NA, nrow = sum(strand$Type == 'row'),  ncol = sum(strand$Type == 'col')  ))
    #refcurve <- ca_procrustes_curve(ref)
    if (display == "ca") {
        plot(ca_procrustes(x$im_seriated))
    } else if (display == "ref") {
        dat <- x$dat
        ord.plot <- ggplot2::ggplot() + 
            ggplot2::geom_point(data = dat, ggplot2::aes(x = CurveIndex, y = Distance, color = Type ), size = 2) + 
            ggplot2::geom_text(data = dat, ggplot2::aes(x = CurveIndex, y = Distance, color = Type ), label = rownames(dat), hjust = "left", nudge_y = 0.0005, size = 3,  angle = 90,  check_overlap = TRUE) + 
            ggplot2::theme_bw() + ggplot2::theme(aspect.ratio = .3, legend.position="none")
        print(ord.plot)
    } else if (display == "im_seriated") {
        plot(x$im_seriated)
        #strand2 <- strand$im_seriated
        #im.Image <- t(strand2[nrow(strand2):1 , ])
        #image(im_Image, col=c('white','black'), xaxt='n', yaxt='n')
    } else {
        stop('Display option must be "ca", "ref", "im_serated"')
    }
}




#' @export 
print.strands <- function(x, ...) {
    cat('List of', length(x), 'strands\n')
}





#' @export 
print.lakhesis <- function(x, ...) {
    cat("Lakhesis analysis of class:", class(x), "\nUse summary() for more information. \n")
}




#' @export 
summary.lakhesis <- function(object, ...) {
    cat("Lakhesis analysis of class:", class(object), "\n")
    cat("\n")
    cat("Ranking of row elements: \n")
    cat(object$row, sep = ", ", fill = TRUE)
    cat("\n")
    cat("Ranking of column elements: \n")
    cat(object$col, sep = ", ", fill = TRUE)
    cat("\n")
    cat("Seriated incidience matrix in $im_seriated\n")
    cat("Coefficients in $coef\n")
    cat("cor_sq = ", cor_sq(object$im_seriated), "\n")
    cat("conc_wrc = ", conc_wrc(object$im_seriated), "\n")
}



#' @export 
plot.lakhesis <- function(x, display = "im_seriated", ...) {
    Strand <- Agreement <- Criterion <- NULL
    lakhcoef <- x$coef
    if (display == "im_seriated") {
        im_seriated <- x$im_seriated
        k_sp <- cor_sq(im_seriated)
        k_wrc <- conc_wrc(im_seriated)
        ttl <- paste(format(nrow(im_seriated))," x ",format(ncol(im_seriated)),"; cor_sq = ",format(round(k_sp,3)), "; conc_wrc = ",format(round(k_wrc,3)), sep = "")
        im_Image <- t(im_seriated[nrow(im_seriated):1 , ])
        graphics::image(im_Image, col=c('white','black'), xaxt='n', yaxt='n', main = ttl)
    } else if (display == "agreement") {
        lakhcoef$Strand <- factor(lakhcoef$Strand, levels = lakhcoef$Strand[order(lakhcoef$Agreement, decreasing = FALSE)]) 
        plot_agreement <- ggplot2::ggplot(lakhcoef, ggplot2::aes(x=Strand, y=Agreement)) + 
            ggplot2::geom_bar(stat = "identity") + ggplot2::theme_bw()
        print( plot_agreement )
    } else if (display == "criterion") {
        lakhcoef$Strand <- factor(lakhcoef$Strand, levels = lakhcoef$Strand[order(lakhcoef$Criterion, decreasing = TRUE)]) 
        plot_criterion <- ggplot2::ggplot(lakhcoef, ggplot2::aes(x=Strand, y=Criterion)) + 
            ggplot2::geom_bar(stat = "identity") + ggplot2::theme_bw()
        print( plot_criterion )
    } else {
        stop("Choose a valid display option: im_seriated, agreement, criterion")
    }
}



#' @export 
plot.incidence_matrix <- function(x, ...) {
    k_wrc <- conc_wrc(x)
    k_sp <- cor_sq(x)
    ttl <- paste(format(nrow(x))," x ",format(ncol(x)),"; cor_sq = ", format(round(k_sp,3)), "; conc_rc = ",format(round(k_wrc,3)), sep = "")
    im_Image <- t(x[nrow(x):1 , ])
    graphics::image(im_Image, col=c('white','black'), xaxt='n', yaxt='n', main = ttl)
}

#' Correspondence Analysis with Procrustes Fitting
#'
#' Fit scores of correspondence analysi on an incidence matrix to those produced by reference matrix which contain an ideal seriation using a Procrustes method (on the reference matrix, see \code{\link[lakhesis]{im_ref}}).  Rotation is determined by minimizing Euclidean distance from each row score to the nearest reference row score. Correspondence analysis is performed using the \code{\link[ca]{ca}} package \insertCite{nenadic_correspondence_2007}{lakhesis}.
#'
#' @param obj An incidence matrix of size n x k.
#' @param symmetric Whether to use standard scores for both rows and columns. Default is \code{TRUE}. Setting this to \code{FALSE} will result in a asymmetric map of standard scores for rows and principal scores for columns.
#' @returns A \code{list} object of class \code{strand} containing the following:
#' * `ref` The Procrustes-fit coordinates of the scores of the reference seriation.
#' * `x` The coordinates of the row standard scores of the data.
#' * `y` The coordinates of the column principal scores of the data.
#' * `x_pr` The Procrustes-fit coordinates of the row standard scores of the data.
#' * `y_pr` The Procrustes-fit coordinates of the column column scores of the data.
#' 
#' @examples 
#' data("quattrofontanili")
#' s <- ca_procrustes(quattrofontanili)
#' # print(s)
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' @importFrom Rdpack reprompt
ca_procrustes <- function(obj, symmetric = TRUE) {
    UseMethod("ca_procrustes")
}
#'
#' @export
ca_procrustes.default <- function(obj, symmetric = TRUE) {
    stop(paste('ca_procrustes cannot take input of class "', class(obj), '" ', sep =''))
}

#' @rdname ca_procrustes
#' @export
ca_procrustes.matrix <- function(obj, symmetric = TRUE) {
    if (sum(is.na(match(rownames(obj), colnames(obj)))) != nrow(obj))  {
        stop("Row names and column names of input matrix be unique.")
    }
    results <- list()

    obj <- obj[sort(rownames(obj)), sort(colnames(obj))]
    transposed <- FALSE
    if (nrow(obj) > ncol(obj)) {
        obj <- t(obj)
        transposed <- TRUE
    }

    if (symmetric == TRUE) {
        ref <- im_ref(obj)

        G.r <- ca::ca(obj)$rowcoord[,1:2]
        G.c <- ca::ca(obj)$colcoord[,1:2]
        R.r <- ca::ca(ref)$rowcoord[,1:2]
        R.c <- ca::ca(ref)$colcoord[,1:2]
    } else {
        ref <- im_ref(obj)
        S <- diag(ca::ca(obj)$sv)
        Sref <- diag(ca::ca(ref)$sv)

        G.r <- ca::ca(obj)$rowcoord[,1:2]
        G.c <- ca::ca(obj)$colcoord %*% S
        G.c <- G.c[,1:2]
        R.r <- ca::ca(ref)$rowcoord[,1:2]
        R.c <- ca::ca(ref)$colcoord %*% Sref
        R.c <- R.c[,1:2]
    }

    #data
    G.rc <- G.r - t(replicate( nrow(G.r) , apply(G.r, 2, mean))) # mean center
    G.rc.rad.max <- max( sqrt(G.rc[,1]^2 + G.rc[,2]^2) ) # rescale
    x.r <- G.rc / G.rc.rad.max

    G.cc <- G.c - t(replicate( nrow(G.c) , apply(G.r, 2, mean))) # mean center with row data
    x.c <- G.cc / G.rc.rad.max  # rescale with row data

    #reference
    R.rc <- R.r - t(replicate( nrow(R.r) , apply(R.r, 2, mean))) # mean center
    R.rc.rad.max <- max( sqrt(R.rc[,1]^2 + R.rc[,2]^2) ) # rescale
    ref_r <- R.rc / R.rc.rad.max

    R.cc <- R.c - t(replicate( nrow(R.c) , apply(R.r, 2, mean))) # mean center with row data
    ref.c <- R.cc / R.rc.rad.max  # rescale with row data


    # procrustes rotation without landmark points

    # minimize rss of one-to-many distance from data to reference curve
    mid <- round(nrow(ref_r)/2)
    ref_mid_rad <- atan2(ref_r[mid,2], ref_r[mid,1])
    rss <- rss_rotation(x.r, ref_r, ref_mid_rad)

    # rotate data using row points
    idx <- which.min(rss)

    theta <- atan2(x.r[idx,2], x.r[idx,1]) - ref_mid_rad
    rotate <- matrix(c(cos(theta), sin(theta), -sin(theta), cos(theta)), nrow = 2, ncol = 2)
    x.r.rot <- x.r %*% rotate
    x.c.rot <- x.c %*% rotate

    results[['ref']] <- ref_r
    if (transposed == FALSE) {
        results[['x']] <- G.r
        results[['y']] <- G.c
        results[['x_pr']] <- x.r.rot
        results[['y_pr']] <- x.c.rot
    } else {
        results[['y']] <- G.r
        results[['x']] <- G.c
        results[['y_pr']] <- x.r.rot
        results[['x_pr']] <- x.c.rot
    }

    class(results) <- c("procrustean", "list")

    return(results)
}

#' @rdname ca_procrustes
#' @export
ca_procrustes.incidence_matrix <- function(obj, symmetric = TRUE) {
    ca_procrustes.matrix(obj, symmetric)
}



#' Seriate Procrustes-Fit CA Scores
#' 
#' Obtain a ranking of row and column scores projected onto a reference curve of an ideal seriation (row and column scores are ranked separately). Scores of correspondence analysis have been fit to those produced by reference matrix contain an ideal seriation using a Procrustes method, projecting them. Rotation is determined by minimizing Euclidean distance from each row score to the nearest reference row score. Correspondence analysis is performed using the \code{\link[ca]{ca}} package \insertCite{nenadic_correspondence_2007}{lakhesis}.
#'
#' @param obj An incidence matrix of size n x k.
#' @param projection Which projection to use:
#'   * \code{ca1} The first axis for CA scores.
#'   * \code{ca2} The second axis for CA scores.
#'   * \code{procrustes1} The first axis of Procrustes-fit CA scores.
#'   * \code{procrustes2} The second axis of Procrustes-fit CA scores.
#'   * \code{curve} The reference curve of an ideal seriation, using Procrustes fitting (this is the default).
#' @param samples Number of samples to use for plotting points along polynomial curve. Default is \code{10^5}.
#' @param symmetric Whether to use standard scores for both rows and columns. Default is \code{TRUE}. Setting this to \code{FALSE} will result in a asymmetric map of standard scores for rows and principal scores for columns.
#' @return A \code{list} of class \code{strand} containing the following:.
#' * \code{$dat} A data frame with the following columns:
#'   * \code{Procrustes1, Procrustes2} The location of the point on the biplot after fitting.
#'   * \code{CurveIndex} The orthogonal projection of the point onto the reference curve, given as the index of the point sampled along \eqn{y = \beta_2 x^2 + \beta_0}. 
#'   * \code{Distance} The squared Euclidean distance of the point to the nearest point on the reference curve.
#'   * \code{Rank} The ranking of the row or column, a range of `1:nrow`` and `1:ncol``.
#'   * \code{Type} Either `row` or `col`.
#'   * \code{sel} Data frame column used in \code{shiny} app to indicate whether point is selected in biplot/curve projection.
#' * \code{$im_seriated} The seriated incidence matrix, of class \code{incidence_matrix}.
#' 
#' @examples
#' data("quattrofontanili")
#' s <- ca_procrustes_ser(quattrofontanili)
#' # print(s)
#' # summary(s)
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' @importFrom Rdpack reprompt
ca_procrustes_ser <- function(obj, projection = "curve", samples = 10^5, symmetric = TRUE) {
    UseMethod("ca_procrustes_ser")
}

#' @rdname ca_procrustes_ser
#' @export
ca_procrustes_ser.incidence_matrix <- function(obj, projection = "curve", samples = 10^5, symmetric = TRUE) {
    ca_procrustes_ser.matrix(obj, projection, samples, symmetric)
}

#' @rdname ca_procrustes_ser
#' @export
ca_procrustes_ser.matrix <- function(obj, projection = "curve", samples = 10^5, symmetric = TRUE) {
    if ((nrow(obj) > 2 ) & (ncol(obj) > 2) ) {
        if (!(projection %in% c("ca1", "ca2", "procrustes1", "procrustes2", "curve"))) {
            message("Invalid projection method.")
        } else {
            obj <- obj[sort(rownames(obj)), sort(colnames(obj))]
            obj_procca <- ca_procrustes(obj, symmetric = symmetric)
            ref_r <- obj_procca$ref
            x_r_rot <- obj_procca$x_pr
            x_c_rot <- obj_procca$y_pr
            x <- ref_r[,1]
            y <- ref_r[,2]

            # polynomial fitting
            xx <- seq(-1, 1, length.out = samples)
            fit <- stats::lm(y ~ x + I(x^2))
            yy <- stats::predict(fit, data.frame(x = xx))
            ref2_r <- as.matrix(cbind(xx,yy))

            x_ <- orth_proj_fit(x_r_rot, ref2_r)
            x_dat <- data.frame(index = x_[,2], x_fit = x_[,3], y_fit = x_[,4], dist = x_[,1])
            rownames(x_dat) <- rownames(obj)

            # col points
            y_ <- orth_proj_fit(x_c_rot, ref2_r)
            y_dat <- data.frame(index = y_[,2], x_fit = y_[,3], y_fit = y_[,4], dist = y_[,1])
            rownames(y_dat) <- colnames(obj)

            if (projection == "curve") {
                r_idx <- x_dat$index
                c_idx <- y_dat$index
            } else if (projection == "procrustes1") {
                r_idx <- x_r_rot[,1]
                c_idx <- x_c_rot[,1]
            } else if (projection == "procrustes2") {
                r_idx <- x_r_rot[,2]
                c_idx <- x_c_rot[,2]
            } else if (projection == "ca1") {
                r_idx <- obj_procca$x[,1]
                c_idx <- obj_procca$y[,1]
            } else if (projection == "ca2") {
                r_idx <- obj_procca$x[,2]
                c_idx <- obj_procca$y[,2]
            }

            r_rank <- rank(r_idx)
            c_rank <- rank(c_idx)

            ord_row1 <- data.frame(CA1 = obj_procca$x[,1], CA2 = obj_procca$x[,2],Procrustes1 = obj_procca$x_pr[,1], Procrustes2 = obj_procca$x_pr[,2], CurveIndex = r_idx, Distance = x_dat$dist, Rank = r_rank)
            rownames(ord_row1) <- rownames(x_dat)
            ord_row1$Type <- factor("row")
            ord_col1 <- data.frame(CA1 = obj_procca$y[,1], CA2 = obj_procca$y[,2],Procrustes1 = obj_procca$y_pr[,1], Procrustes2 = obj_procca$y_pr[,2], CurveIndex = c_idx, Distance = y_dat$dist, Rank = c_rank)
            rownames(ord_col1) <- rownames(y_dat)
            ord_col1$Type <- factor("col")
            ranking <- rbind(ord_row1,ord_col1)
            ranking$sel <- FALSE

            R <- rownames(x_dat)[order(r_rank)]
            C <- rownames(y_dat)[order(c_rank)]

            im_seriated <- obj[R, C]
            class(im_seriated) <- c("incidence_matrix", "matrix", "array")
            strand <- list(dat = ranking, im_seriated = im_seriated, method = projection)
            class(strand) <- c("strand", "list")
            return(strand)
        }
    } else {
        message("Insufficient rows and columns for correspondence analysis.")
    }
}

#' Optimality Criterion: Squared Correlation
#' 
#' Treating each incidence of 1 in an element \eqn{(i,j)} of a seriated matrix as an \eqn{(x,y)} point, computes the squared correlation coefficient \insertCite{@see @mccormick_identification_1969, 147-148}{lakhesis}.
#' 
#' @param obj A seriated binary matrix.
#' @returns Spearman's rank correlation coefficient.
#' 
#' @examples
#' data("quattrofontanili")
#' cor_sq(quattrofontanili)
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' @importFrom Rdpack reprompt
cor_sq <- function(obj) {
    UseMethod("cor_sq")
}

#' @rdname cor_sq
#' @export 
cor_sq.matrix <- function(obj) {
    x <- c()
    y <- c()
    for (i in 1:nrow(obj)) {
        yi <- which(obj[i, ] == 1)
        xi <- rep(i, length(yi))
        x <- c(x, xi)
        y <- c(y, yi)
    }
    r_ <- ( stats::cor(x,y, method = "pearson")  )^2
    return(r_)
}

#' @rdname cor_sq
#' @export
cor_sq.incidence_matrix <- function(obj) {
    cor_sq.matrix(obj)
}



#' Spearman Correlation Squared
#' 
#' The square of Spearman's rank correlation coefficient applied to two rankings \insertCite{spearman_proof_1904}{lakhesis}. Rows with `NA` values are automatically removed.
#' 
#' @param r1,r2 Two vectors of paired ranks.
#' @returns The square of Spearman's rank correlation coefficient with NA values removed.
#' 
#' @examples
#' # e.g., for two partial seriations:
#' x <- c(1, 2, 3, 4, NA, 5, 6, NA, 7.5, 7.5, 9)
#' y <- c(23, 17, 19, NA, 21, 22, 25, 26, 27, 36, 32)
#' spearman_sq(x, y)
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' @importFrom Rdpack reprompt
spearman_sq <- function(r1, r2) {
    UseMethod("spearman_sq")
}

#' @rdname spearman_sq
#' @export 
spearman_sq.numeric <- function(r1, r2) {
    dat <-  stats::na.omit( data.frame(r1,r2) )
    r <- (stats::cor(dat)[1,2])^2 
    return(r)
}



#' Optimality Criterion: Kendall-Doran (Column) Concentration
#'
#' The Kendall-Doran measure of concentration \insertCite{kendall_statistical_1963,doran_computer_1971}{lakhesis}. In a seriated matrix, this function computes the total number cells between the first and last non-zero value, column by column.
#' 
#' @param obj A seriated binary matrix.
#' @returns The measure of concentration.
#' 
#' @examples 
#' data("quattrofontanili")
#' conc_c(quattrofontanili)
#' 
#' @references
#'   \insertAllCited{}
#' 
#' @export
#' @importFrom Rdpack reprompt
conc_c <- function(obj) {
    UseMethod("conc_c")
}

#' @rdname conc_c
#' @export
conc_c.matrix <- function(obj) {
    conc <- numeric(ncol(obj))
    for (j in 1:ncol(obj)) {
        conc[j] <- max(which(obj[,j] != 0)) - min(which(obj[,j] != 0)) + 1
    }
    return(sum(conc))
}

#' @rdname conc_c
#' @export
conc_c.incidence_matrix <- function(obj) {
    conc_c.matrix(obj)
}


#' Optimality Criterion: Weighted Row-Column Concentration
#'
#' Extends the Kendall-Doran (column) measure of concentration (see \code{\link[lakhesis]{conc_c}}) to include rows and then weights the total measure by the total sum of values in the matrix.
#' 
#' @param obj A seriated binary matrix.
#' @returns The weighted row-column coefficient of concentration.
#' 
#' @examples 
#' data("quattrofontanili")
#' conc_wrc(quattrofontanili)
#' 
#' @export
conc_wrc <- function(obj) {
    UseMethod("conc_wrc")
}

#' @rdname conc_wrc
#' @export
conc_wrc.matrix <- function(obj) {
    nu <- sum(obj)
    k <- ( conc_c(obj) + conc_c(t(obj)) ) / (2 * nu)
    return(k)
}

#' @rdname conc_wrc
#' @export
conc_wrc.incidence_matrix <- function(obj) {
    conc_wrc.matrix(obj)
}



#' Evaluating Element Fit
#'
#' Performs a goodness-of-fit test on individual row and column elements using deviance, using a quadratic-logistic model to fit row and column occurrences. In the case of perfect separation of 0/1 values, an `NA` value is assigned. Results are reported as \eqn{p} values for each row and column.
#' 
#' @param obj A seriated binary matrix.
#' @returns A \code{list} containing results in data frames for row and column elements:
#' 
#' * `RowFit` a data frame containing
#'   * `id` Row element
#'   * `p.val` \eqn{p} values of the row elements
#' * `ColFit` a data frame containing
#'   * `id` Column element
#'   * `p.val` \eqn{p} values of the column elements
#' 
#' @examples 
#' data("quattrofontanili")
#' element_eval(quattrofontanili)
#' 
#' @export
element_eval <- function(obj) {
    UseMethod("element_eval")
}


#' @rdname element_eval
#' @export
element_eval.matrix <- function(obj) {
    dev.c <- numeric(ncol(obj))
    dev.c[] <- NA
    for (j in 1:ncol(obj)) {
        conc_j <- max(which(obj[,j] != 0)) - min(which(obj[,j] != 0)) + 1
        conc_js <- conc_j / sum(obj[,j])
        if (conc_js != 1) {
            x <- 1:length(obj[,j])
            y <- obj[,j]
            suppressWarnings({
            fit <- stats::glm(y ~ x + I(x^2), family = stats::binomial(link = "logit"))
            })
            dev.res <- fit$deviance
            dev.nul <- fit$null.deviance
            df.res <- fit$df.residual
            df.nul <- fit$df.null

            pval <- 1 - stats::pchisq(dev.nul - dev.res, df.nul - df.res)
            dev.c[j] <- pval
        } 
    }
    ColFit = data.frame(id = colnames(obj), p.val = dev.c)
    ColFit <- ColFit[order(ColFit$p.val, decreasing = TRUE) , ]

    dev.r <- numeric(nrow(obj))
    dev.r[] <- NA
    for (i in 1:nrow(obj)) {
        conc_i <- max(which(obj[i,] != 0)) - min(which(obj[i,] != 0)) + 1
        conc_is <- conc_i / sum(obj[i,])
        if (conc_is != 1) {
            x <- 1:length(obj[i,])
            y <- obj[i,]
            suppressWarnings({
            fit <- stats::glm(y ~ x + I(x^2), family = stats::binomial(link = "logit"))
            })
            dev.res <- fit$deviance
            dev.nul <- fit$null.deviance
            df.res <- fit$df.residual
            df.nul <- fit$df.null

            pval <- 1-stats::pchisq(dev.nul - dev.res, df.nul - df.res)
            dev.r[i] <- pval
        } 
    }
    RowFit <- data.frame(id = rownames(obj), p.val = dev.r)
    RowFit <- RowFit[order(RowFit$p.val, decreasing = TRUE) , ]

    results <- list(RowFit = RowFit, ColFit = ColFit)
    class(results) <- c("list")
    return(results)
}

#' @rdname element_eval
#' @export
element_eval.incidence_matrix <- function(obj) {
    element_eval.matrix(obj)
}

#' Strand Extract
#' 
#' From a \code{list} of strands produced by \code{\link[lakhesis]{ca_procrustes_ser}}, extract two matrices containing the ranks of the rows and columns. The row/column elements are contained in the rows, and the strands are contained in the columns. NA values are entered where a given row/column element is missing from that strand.
#' 
#' @param strands A \code{list} of class \code{strands}.
#' 
#' @return A list of two matrices: 
#' * \code{Row} A matrix of the ranks of the row elements.
#' * \code{Col} A matrix of the ranks of the column elements.
#' 
#' @examples
#' data("quattrofontanili")
#' data("qf_strands")
#' strand_extract(qf_strands)
#' 
#' @export
strand_extract <- function(strands) {
    UseMethod("strand_extract")
}  

#' @export
strand_extract.strands <- function(strands) {
    obj <- strands[[1]]$im_seriated
    for (i in 2:length(strands)) {
        obj <- im_merge(obj, strands[[i]]$im_seriated)
    }
    if (sum(is.na(match(rownames(obj), colnames(obj)))) != nrow(obj))  {
        stop("Strands improperly produced. Row names and column names of input matrix be unique.")
    }

    rowranks <- matrix(NA, nrow = nrow(obj),  ncol = length(strands)) 
    rownames(rowranks) <- rownames(obj)

    colranks <- matrix(NA, nrow = ncol(obj),  ncol = length(strands)) 
    rownames(colranks) <- colnames(obj)

    for (i in 1:length(strands)) {
        strand <- strands[[i]]$dat

        strand.r <- strand[strand$Type == "row", ]
        strand.c <- strand[strand$Type == "col",]

        rowranks[ match(rownames(strand.r), rownames(rowranks)) , i] <- strand.r$Rank
        colranks[ match(rownames(strand.c), rownames(colranks)) , i] <- strand.c$Rank
    }

    rowranks <- rowranks[rowSums(is.na(rowranks)) != ncol(rowranks), ]
    colranks <- colranks[rowSums(is.na(colranks)) != ncol(colranks), ]

    results <- list()
    results[["Row"]] <- rowranks
    results[["Col"]] <- colranks
    return(results)
}



#' Suppress Element from Strands
#' 
#' Given a \code{list} of \code{strands} produced by correspondence analysis with Procrustes fitting (\code{\link[lakhesis]{ca_procrustes_ser}}), remove one or more row or column elements, re-seriating each strand. This generates a new list of strands that exclude the specified elements. If a resulting strand lacks sufficient points to perform correspondence analysis, that strand is deleted in the output.
#' 
#' @param strands A \code{list} of class \code{strands}.
#' @param elements A vector of one or more row or column ids to suppress.
#' 
#' @return A list of the strands.
#' 
#' @examples
#' data("qf_strands")
#' strand_suppress(qf_strands, "QF II 15-16")
#' 
#' strand_suppress(qf_strands, c("QF II 15-16", "I", "XIV"))
#' 
#' @export
strand_suppress <- function(strands, elements) {
    UseMethod("strand_suppress")
}

#' @rdname strand_suppress
#' @export
strand_suppress.strands <- function(strands, elements) {
    strand_suppress.default(strands, elements)
}

#' @rdname strand_suppress
#' @export
strand_suppress.default <- function(strands, elements) {
    new <- list()
    obj <- strands[[1]]$im_seriated
    for (s in 2:length(strands)) {
        obj <- im_merge(obj, strands[[s]]$im_seriated)
    }
    j <- 1
    for (s in 1:length(strands)) {
        strand <- strands[[s]]$dat
        rows <- rownames(strand)[ !(rownames(strand) %in% elements) & (strand$Type == "row") ]
        cols <- rownames(strand)[ !(rownames(strand) %in% elements) & (strand$Type == "col") ]
        obj_copy <- obj
        obj_copy <- obj_copy[rows, ]
        obj_copy <- obj_copy[ ,cols]
        obj_copy <- obj_copy[rowSums(obj_copy) != 0 , ]
        obj_copy <- obj_copy[ , colSums(obj_copy) != 0]
        newstrand <- NULL
        suppressWarnings({
            newstrand <- ca_procrustes_ser(obj_copy)
        })
        if (!is.null(newstrand)) { 
           new[[j]] <- newstrand
           j <- j + 1
        }
    }
    class(new) <- c("strands", "list")
    return(new)
}




#' Add Strand to List of Strands
#' 
#' Given a list of strands, remove a row or column element and re-run seriation by correspondence analysis with Procrustes fitting (\code{\link[lakhesis]{ca_procrustes_ser}}) to generate a new list of strands that exclude the specified elements. If the resulting strand lacks sufficient points to perform correspondence analysis, that strand is deleted in the output.
#' 
#' @param strand An object of class \code{strand} returned by \code{\link[lakhesis]{ca_procrustes_ser}}.
#' @param strands A \code{list} of strands.
#'
#' @return A \code{list} of class \code{strands}.
#' 
#' @export 
strand_add <- function(strand, strands) {
    UseMethod("strand_add")
}

#' @rdname strand_add
#' @export
strand_add.strand <- function(strand, strands) {
    strands[[length(strands) + 1]] <- strand
    class(strands) <- c("strands", "list")
    return(strands)
}


#' Create List of Strands
#' 
#' Given one or more individual strand objects, create a single \code{list} of class \code{strands}.
#' 
#' @param strands A \code{list} of strands.
#'
#' @return A \code{list} of class \code{strands}.
#' 
#' @export 
strands_create <- function(strands) {
    UseMethod("strands_create")
}

#' @rdname strands_create
#' @export
strands_create.list <- function(strands) {
    class(strands) <- c("strands", "list")
    return(strands)
}



#' Create Strand Object from Seriated Incidence Matrix
#' 
#' Given a seriated incidence matrix with unique row and column names, create a \code{strand} object.
#' 
#' @param obj A \code{list} of strands.
#' @param method The method used to create the strand (optional).
#'
#' @return A \code{list} of class \code{strands}.
#' 
#' @export 
strand_create <- function(obj, method = NULL) {
    UseMethod("strand_create")
}

#' @rdname strand_create
#' @export
strand_create.matrix <- function(obj, method = NULL) {

    ord_row <- data.frame(Rank = 1:nrow(obj))
    ord_row$Type <- factor("row")
    rownames(ord_row) <- rownames(obj)
    ord_col <- data.frame(Rank = 1:ncol(obj))
    ord_col$Type <- factor("col")
    rownames(ord_col) <- colnames(obj)

    ranking <- rbind(ord_row,ord_col)
    ranking$sel <- FALSE

    class(obj) <- c("incidence_matrix", "matrix", "array")

    res <- list(dat = ranking, im_seriated = obj, method = method)
    class(res) <- c("strand", "list")
    return(res)
}

#' @rdname strand_create
#' @export
strand_create.incidence_matrix <- function(obj, method = NULL) {
    strand_create.matrix(obj, method)
}

#' Lakhesis Calculator
#'
#' Launch Lakhesis Calculator, a graphical interface to explore binary matrices via correspondence analysis, select potentially well-seriated sequences, and perform consensus seriation. Interface is made with \code{ggplot2}, \code{shiny}, \code{shinydashboard}, and \code{bslib} \insertCite{wickham_ggplot2:_2016,chang_shiny_2024,chang_shinydashboard_2021,sievert_bslib_2024}{lakhesis}. 
#' 
#' Input is done in the calculator, via a "long" format a two-column \code{.csv} file giving pairs of row and column incidences. See \code{\link[lakhesis]{im_read_csv}} for details. Conversion of a pre-existing incidence matrix to long format can be performed with \code{\link[lakhesis]{im_long}}.
#' 
#' Results can be downloaded from the calculator as an \code{.rds} file containing a \code{list} of the following:
#' 
#' * `consensus` The consensus seriations, diagnostic coefficients of agreement and optimality criterion, and seriated incidence matrix.
#' (\code{\link[lakhesis]{lakhesize}}).
#' * `strands` The strands selected by the investigator.
#' 
#' @returns Opens the Lakhesis Calculator. 

#' @references
#'   \insertAllCited{}
#' 
#' @export
#' @importFrom Rdpack reprompt
LC <- function() {
    shiny::runApp(appDir = system.file('app', package = 'lakhesis') )
}

#' Quattro Fontanili
#'
#' The seriation of tombs from necropoleis at Veii, primarily Quattro Fontanili, but also Valle la Fata, Vaccareccia, and Picazzano, in southern Etruria, established by \insertCite{close-brooks_veii_1979;textual}{lakhesis}.
#'
#' @format
#' A seriated incidence matrix of 81 rows (tombs) and 82 columns (types).
#' \describe{
#' Data entered from \insertCite{close-brooks_veii_1979;textual}{lakhesis}, an English translation of the authors' original publication in \emph{Notizie degli Scavi} (1963). Descriptions of types may be found in that paper.
#' }
#' @usage data("quattrofontanili")
#' @examples
#' data("quattrofontanili")
#' print(quattrofontanili)
#' 
#' @references
#'   \insertAllCited{}
"quattrofontanili"





#' Quattro Fontanili - Strands
#'
#' Three seriated strands selected from \code{\link[lakhesis]{quattrofontanili}} data, identified by the package author as an example for the documentation of functions.
#'
#' @format
#' A \code{strands} object containing strands output by \code{\link[lakhesis]{ca_procrustes}}.
#' 
#' @usage data("qf_strands")
#' @examples
#' data("qf_strands")
#' print(qf_strands)
#' 
"qf_strands"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @useDynLib lakhesis
#' @importFrom Rcpp sourceCpp
rss_rotation <- function(x_r, ref_r, ref_mid_rad) {
    .Call('_lakhesis_rss_rotation', PACKAGE = 'lakhesis', x_r, ref_r, ref_mid_rad)
}

#' @useDynLib lakhesis
#' @importFrom Rcpp sourceCpp
orth_proj_fit <- function(x_r_rot, ref_r) {
    .Call('_lakhesis_orth_proj_fit', PACKAGE = 'lakhesis', x_r_rot, ref_r)
}

---
title: "A Guide to Lakhesis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{A Guide to Lakhesis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette provides a guide to the R package `lakhesis`. Please refer to the paper, "Lakhesis: Consensus Seriation via Iterative Regression of Partial Rankings for Binary Data," for details.

## Introduction

Seriation can be accomplished using a variety of methods. The main goals of the `lakhesis` package, which focus on binary matrices (i.e., presence/absence, 0/1), are to provide:

1. an interactive graphical interface for investigators to select their own well seriated data and perform seriation via correspondence analysis;
2. the means to establish a single consensus seriation from multiple, partial seriations via a critical coefficient of optimality.

While there are already a number of approaches and tools to perform seriations, `lakhesis` uses a technique of fitting CA scores to those of a curve produced by an ideal, well seriated "reference matrix" via a Procrustes method (center, scale, and rotate).

Since well seriated matrices are not a given for every data set, it is expedient for users to select their own seriations quickly via an interactive graphical interface, rather than slicing matrices and re-running commands via the console. Investigators can select multiple seriations, moreover, which can then be harmonized into a single consensus seriation.

The `lakhesis` package, named for the ancient Greek goddess who measured the thread of fate, provides this functionality, to explore and select well seriated sequences as "strands" that are then "threaded" together into a single consensus seriation. The package can either be installed from [CRAN](https://CRAN.R-project.org/package=lakhesis), or from [GitHub](https://github.com/scollinselliott/lakhesis), the latter of which hosts the most recent development version. To install from GitHub, the library `devtools` is needed:
```{r, eval = FALSE}
library("devtools")
install_github("scollinselliott/lakhesis", dependencies = TRUE, build_vignettes = TRUE) 
```

This vignette showcases the primary functions of the package along with the Lakhesis Calculator, a `shiny` app whose interface offers the means to perform and critically examine seriated sequences.

## Getting Started: Formatting Data

The Calculator is designed to import `csv` files directly from the user's file directory. The `csv` file must be formatted in a "long" format where row and column incidences are listed in rows, pair by pair. To convert data that is already in the form of a matrix, the `im_long()` function can used.  The `"quattrofontanili"` data included in the package can serve as an example. This data set comprises a matrix of 81 rows and 82 columns, recording the incidence of a particular artifact-type (column) in a tomb (row) from Early Iron Age necropoleis in Southern Etruria (see `?quattrofontanili`).
```{r}
library("lakhesis")
data("quattrofontanili")
qf_long <- im_long(quattrofontanili)
```
The `data.frame` `qf_long` can then be exported to a `csv` file using the `write.table()` function, in order to avoid writing row and column headers:
```{r, eval = FALSE}
write.table(qf_long, file = "qf.csv", row.names = FALSE, col.names = FALSE, sep = ",")
```
The file `qf.csv` is then ready to be uploaded into the Lakhesis Calculator.

### Pooling Observations

If one has multiple data sets to pool together, it is simply a matter of adding supplementary rows to the `csv` file. The `im_merge()` function assists with creating a single incidence matrix from two different incidence matrices, overriding any 0 values with 1s in the case of shared row and/or column elements. The merged matrix can then be converted using `im_long()` and exported, as above.

```{r, eval = FALSE}
qf1 <- quattrofontanili[1:20, 1:40]
qf1 <- qf1[rowSums(qf1) != 0, colSums(qf1) != 0]

qf2 <- quattrofontanili[30:50, 20:60]
qf2 <- qf2[rowSums(qf2) != 0, colSums(qf2) != 0]

im_merge(qf1, qf2)
```

## Procrustes-Fit Correspondence Analysis

The Lakhesis Calculator uses correspondence analysis (CA), a popular method used for seriation especially in the form of detrended correspondence analysis (DCA). Unlike DCA, `lakhesis` seriates observations according to a "reference curve," since the plot of  principal scores of well seriated rows and columns will have the form of an arch or horseshoe. Principal scores of the data are fit to the curve of the reference matrix using a Procrustes method (center, scale, rotate), with axes are properly labelled "Procrustes1" and "Procrustes2" since they no longer represent the chi-squared distance between row and columnt elements, separately. Procrustes fitting is done without landmark points by minimizing the squared Euclidean distance of scores to those of the reference matrix. 

This task is accomplished with the function `ca_procrustes()`, which takes an incidence matrix as an input and returns an object `procrustean` that contains the reference and data scores. The reference matrix is generated internal to `ca_procrustes()` using the `im_ref()` function. The `plot` function automatically illustrates the fit of the points to the reference curve:
```{r,  fig.align = 'center'}
data("quattrofontanili")
qf_caproc <- ca_procrustes(quattrofontanili)
class(qf_caproc)
print(qf_caproc)
plot(qf_caproc)
```

In order to determine a seriation for both the row and column points, scores are projected onto the quadratic model which fits the reference points. Then, their ranking is taken from one end of the curve to another. The function `ca_procrustes_ser()` takes an incidence matrix as input, and produces as output a `strand` object, which comprises a `list` that includes data regarding the row and column score points, distance, and rankings, as well as the seriated matrix according those rankings.
```{r}
qf_ser <- ca_procrustes_ser(quattrofontanili)
class(qf_ser)
print(qf_ser)
summary(qf_ser)
```

Three different plot types are available for a `strand` object, set in the `plot()` function via the additional `display` argument:

* `"ca"` produces a Procrustes-fit CA plot as above, with axes $\text{Procrustes1}$ and $\text{Procrustes2}$. This is the default setting.
* `"ref"` produces a plot showing the distance of each row and column point from the reference curve. The indices of the reference curve are shown on the $x$ axis while the distance of each point from the curve is shown on the $y$ axis.
* `"im_seriated"` produces a matrix plot of the seriated incidence matrix, showing the measure of concentration $\kappa$ (on which see below).

```{r, fig.align = 'center'}
plot(qf_ser, display = "ca")
plot(qf_ser, display = "ref")
plot(qf_ser, display = "im_seriated")
```

The resulting incidence matrix may be extracted from the `strand` by calling `$im_seriated`:

```{r}
quattrofontanili.caproc <- qf_ser$im_seriated
```

## Lakhesis Analysis

In the process of exploring matrices for seriated sequences, an investigator is liable to come across many sequences which appear to be well seriated. The question then arises as to how to harmonize or reconcile several partial, seriated sequences, some or many of which may disagree. This problem is non-trivial, analogous to other situations such as e.g. ranked-choice or preference voting, although not strictly identical.

The strategy adopted here, called a "Lakhesis" method, is to iterate simple linear regressions in an agglomerative fashion. The procedure consists of the followings steps:

1. To initialize, perform pairwise linear regression on all strands (each strand must have at least 4 joint elements with at least one other strand), for row and column elements separately.
    a. For any two strands, select one as the dependent ($y$) and another as the independent ($x$) variate.
    b. Remove all elements for there is an `NA` value in either strand.
    c. Use the regression line, $f(x) = \beta_1 x + \beta_0$, to project the ranks of the independent variate onto the dependent (thereby supplying rankings for any `NA` values in the dependent variate).
    d. If $y \neq f(x)$, the mean of $y$ and $f(x)$ is used for that element.
    e. The values of the dependent and regressed independent variates are re-ranked.
2. The concentration coefficient $\kappa$ (on which see below) is computed for each pairwise regression.
3. The pair of strands is chosen that elicits the lowest concentration measure. The regression from this pair is then selected as the dependent variate for the next iteration.
4. Perform pairwise regression with all remaining strands as the independent variate and the regression from the previous iteration, repeating Steps 1b-3.
6. Repeat Step 4 until all strands have been regressed, resulting in a single consensus seration.

Linear regression has its virtue in that partial rankings which may be identical in their relative sequence but which may run in reverse will be regressed into the same order, merely with a negative slope.

Given that the order in which strands are regressed has a bearing on the resulting consensus seriation, the use of an optimality measure to determine that order is helpful. Especially if the number of strands were to exceed 10, all possible permutations of the order of strand regression could not be evaluated in a reasonable amount of time. Similarly, PCA (which was initially explored as a method of harmonizing rankings) would only work as a method of ranking imputation if there were at least four elements jointly shared across all strands, a restriction which was undesirable given a potential need to perform consensus seration on a large set of data in piecemeal fashion.

### The `lakhesize()` Function

Computing a consensus seriation for a set of strands via the steps above is performed by the `lakhesize()` function, which takes as its input a `strands` object (i.e., a `list` of individual `strand`s produced by `ca_procrustes_ser()`).

The `lakhesize()` function has the following options:

* `pbar` Turns the progress bar on or off. Default is `TRUE`.

A predefined `qf_strands` data object is included in the package to illustrate functionality of the `lakhesize()` function, comprised of three strands from the `quattrofontanili` data:

```{r}
print(qf_strands)
L <- lakhesize(qf_strands, pbar = FALSE)
summary(L)
```

The output is contained in a `lakhesis` object, which includes the following objects as described in the `summary()` above:

* seriated sequences of the row and column elements according to the consensus seriation of the input strands
* the seriated incidence matrix
* coefficients of concentration and agreement (on which see below)

## Optimality Criteria

The `lakhesis` package employs three methods to critically evaluate a seriated incidence matrix.

### Spearman Correlation


### Concentration (Weighted Row-Column)

In this package, a measure of concentration has been defined, $\kappa$, which is a loss measure with a minimum of 1. Concentration is based on the number of non-contiguous values of 1 attested in the seriated matrix across both columns and rows. The intervals of the index from the lowest-ranked 1 and that of the highest ranked 1 is summed for each row and column, and then divided by twice total sum of the matrix. A seriated matrix that has perfect concentration will thus have $\kappa = 1$, and should there occur 0s in between values of 1, the measure of $\kappa$ will increase. This is performed using the function `conc_wrc()`, whose documentation contains more information, which takes as input an incidence matrix.

Here, we can can create an object that contains the matrix according to the consensus seriation via Lakhesis:
```{r}
quattrofontanili.L <- L$im_seriated
```
Then, one can assess the concentration measure $\kappa$ for each seriated matrix, first according to the original sequence, second according to Procrustes-fit CA, and third according to the Lakhesis method employed for three investigator-selected strands:
```{r}
# the original seriation
conc_wrc(quattrofontanili)
# using procrustes-fit ca
conc_wrc(quattrofontanili.caproc)
# using lakhesis
conc_wrc(quattrofontanili.L)
```
Lower concentration measures inidicate more optimal seriations. Here, consensus via Lakhesis has arrived at a more optimal seriation according to the concentration principle than either just running one seriation using Procrustes-fit CA or the original sequence.

### Agreement ($\rho_r^2 \rho_c^2$)

The agreement of each strand's ranking with that of the resulting consensus seriation also serves as a useful diagnostic to determine if any one strand is in severe disagreement and should be removed before re-running Lakhesis. Agreement is defined as the product of Spearman's rank correlation coefficient squared for the row and column elements between one seriated matrix and another. In the `lakhesize()` function, agreement is computed for each strand's seriation with respect to the consensus seriation, using the function `spearman_sq()`, which removes `NA` values automatically and takes a numerical input of the rankings.

### Deviance Test for Goodness-of-Fit

The use of a quadratic-logistic model for determining seriations has a long history in ecology, but as a measure for optimality it poses a challenge in that an optimally concentrated seriation would elicit `NA` results for a logistic regression, owing to the perfect separation of 0/1 values in the columns and rows.

While a deviance-based test for goodness-of-fit is often performed for (nested) models upon the same data, here it is used as an exploratory method to evaluate which specific row and column elements might not fit well, and which one may wish to exclude when prior to performing correspondence analysis. The function `element_eval()` takes a seriated matrix as its input and will return a `list` of two `data.frame`s sorted from highest to lowest in terms of the resulting $p$ value from a deviance test using the quadratic-logistic model. Any rows or columns which exhibit perfect separation are first assigned a result of `NA`. Then, the deviance test is run on the remaining rows and columns. Elements which contain higher $p$ values will have a less optimal fit. 
```{r}
qf_eval <- element_eval(quattrofontanili)
head(qf_eval$RowFit)
head(qf_eval$ColFit)
```

## Plotting Results

Plotting the results of `lakhesize()` can be accomplished by stipulating different options for the `display` argument in a `plot()` function, taking a `lakhesis` object as input.

* `"im_seriated"` will produce a matrix plot of the consensus seriation, also displaying the concentration measure $\kappa$. This is the default.
* `"agreement"` will produce a bar graph showing the measures of agreement of each strand with the consensus. Higher values indicate better agreement.
* `"criterion"` will produce a bar graph of each strand's criterion. Lower values indicate more optimal seriations.

```{r, fig.align = 'center'}
plot(L, display = "im_seriated") 
plot(L, display = "agreement") 
plot(L, display = "criterion") 
```

These plots are incorporated into the Lakhesis Calculator, and are generated automatically.

## Lakhesis Calculator

The commands above can all be used in the console. That said the functionality of the `lakhesis` package is best articulated via the interactive Lakhesis Calculator, written using `shiny`, which is initialized by the function `LC()`:
```{r, eval = FALSE}
LC()
```
This executes a `shinyApp()` function that will launch the calculator in the system browser. In the browser window, a sidebar is located on the left, while four panels are visible in the main dashboard:

* Seriation Explorer
* Consensus Seriation
* Criteria
* Modify

The calculator initializes with an ideally seriated reference matrix $20 \times 20$ in size, with row elements labeled "R" and column elements labeled "C." This reference matrix is produced using the function `im_ref()`:
```{r, eval = FALSE}
im_ref( matrix(NA, nrow = 20, ncol = 20) )
```

### Uploading Data

Before selecting the dataset to upload, one can choose whether or not the Calculator will automatically remove row and column elements that are attested only once. This is done in the top left sidebar with the radio button under **Before Uploading**. The option **Use All Data** will use the full data when generating the incidence matrix within the calculator. The option **Remove Hapax** will remove all rows and columns which have only one attestation, since these instances do not establish connective relationships with other rows and columns. This may be desired if one has a large matrix.

To upload data, click the **Browse...** button at the top of the sidebar under the **Choose CSV file** heading, and select the `qf.csv` file  exported earlier in the vignette.

### Identifying and Selecting Seriated Sequences

The plot in the upper left panel, Seriation Explorer, will automatically adjust from the initial default to display the correspondence analysis (CA) plot which has been Procrustes-fit to its reference curve. The Seriation Explorer contains two tabs, each of which display the same results, but in two different plot formats:

* The **CA-Procrustes Plot** shows the typical CA plot, whose scores have been moved to fit the reference curve displayed in gray.
* The **Curve Plot** shows the projections of those same points onto the reference curve, from one end to another. Values along the $x$-axis are the indices of points, which are used to derive a ranking of points, from left to right. The distance along the $y$-axis gives the squared Euclidean distance from the point to its orthogonal projection on the reference curve.

The `ca_procrustes_ser()` function is run automatically. The Seriation Explorer is an interactive panel, from which the investigator can select points, to re-run and re-fit correspondence analysis. Points are selected using either the CA plot or the curve reference plot with the mouse cursor via the `brushedPoints()` function in `shiny`. While the CA plot is useful for most selections, if the investigator wants to only include points which lie nearest to the reference curve, the curve plot facilitates selection of points within a chosen distance to the curve.

When points are selected, their color should automatically change. Note that the selection tool gathers points, and so if a particular shape of a cluster or grouping is to be selected, the brush tool can be used repeatedly to select multiple points. In order to deselect and start over, one double-clicks on the plot.

After selecting points of interest, the investigator then clicks **Recompute with Selection**. Correspondence analysis and Procrustes fitting will then be performed again on a matrix constructed only of the selected row and column elements, omitting any rows and columns which contain all zeros. The plots will automatically refresh. When one believes they have found a well seriated sequence, one clicks on the **Save Seriation as Strand** button. This will log the observed seriation as a `strand` inside a `strands` object (no changes occur to the plots), using the `strand_add()` function.

### Creating a Consensus Seriation

After one has selected at least two strands, a consensus seriation can be produced. To do so, click on the **Lakhesize Strands** button in the sidebar, which will perform function `lakhesize()` on the selected strands. Plots will automatically appear in Consensus Seriation and Criteria panels. In the Consensus Seriation panel is shown a matrix plot that gives the size and concentration coefficient of the matrix according to the consensus (not all row and column elements in the input data must be included). In the bottom left Criteria panel, the coefficients of agreement and concentration of each strand are displayed on separate tabs. 

### Modifying Data

#### Deleting Strands

In the process of performing exploratory seriations, it might become clear that one or more strands are poorly seriated, and hence eliminating them might improve the resulting seriated matrix. 

On the Modify panel to the bottom right, one can use the input box labelled **Delete Strand (Indices Will Resort Automatically, Undo Will Re-Add as Last Strand)**, to type in the number of strand to be deleted. Pressing the **Delete** button will remove that strand and automatically readjust the the consensus seriation. The numbering of the strands automatically adjusts. One can add the deleted strand back in, by pressing the **Undo** button. The deleted strand is returned to the end of the strand list, and again the consensus seriation is adjusted automatically.

#### Suppressing Row/Column Elements

There likewise might be a consistent row or column element which appears to be poorly seriated, which the investigator would like to suppress from selection. Following the execution of **Lakhesize Strands**, one can press the **Run Deviance Test** button on the sidebar. The tabs marked "Deviance (Rows)" and "Deviance (Columns)" are now populated with a table of the top ten row and column elements which have the highest $p$~values per the hypothesis test. In the Modify panel, any elements can be entered into the input boxes marked **Select Rows/Columns to Suppress,** either by selecting them from a list or by typing. The Seriation Explorer will automatically remove the row or column elements from the plot. Suppressing row or column elements does not affect previously selected strands. To restore elements, simply delete them from the suppression list in the Modify panel.

### Exporting Results

Exporting results can be done at any time following the command of **Lakhesize Strands**, either in the course of analysis or at the end. The **Export Data** button will download a `list` object in `rds` format which contains the following objects:

* `$consensus` An object of the `lakhesis` class that contains the information on the consensus seriation (see above on the `lakhesize()` function)
* `$strands` An object the `strands` class, the inputs used to produce the `lakhesis` class object

The default filename is automatically timestamped. The `rds` file can then be read back into R for further analysis. 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/criterion.R
\name{cor_sq}
\alias{cor_sq}
\alias{cor_sq.matrix}
\alias{cor_sq.incidence_matrix}
\title{Optimality Criterion: Squared Correlation}
\usage{
cor_sq(obj)

\method{cor_sq}{matrix}(obj)

\method{cor_sq}{incidence_matrix}(obj)
}
\arguments{
\item{obj}{A seriated binary matrix.}
}
\value{
Spearman's rank correlation coefficient.
}
\description{
Treating each incidence of 1 in an element \eqn{(i,j)} of a seriated matrix as an \eqn{(x,y)} point, computes the squared correlation coefficient \insertCite{@see @mccormick_identification_1969, 147-148}{lakhesis}.
}
\examples{
data("quattrofontanili")
cor_sq(quattrofontanili)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{quattrofontanili}
\alias{quattrofontanili}
\title{Quattro Fontanili}
\format{
A seriated incidence matrix of 81 rows (tombs) and 82 columns (types).
\describe{
Data entered from \insertCite{close-brooks_veii_1979;textual}{lakhesis}, an English translation of the authors' original publication in \emph{Notizie degli Scavi} (1963). Descriptions of types may be found in that paper.
}
}
\usage{
data("quattrofontanili")
}
\description{
The seriation of tombs from necropoleis at Veii, primarily Quattro Fontanili, but also Valle la Fata, Vaccareccia, and Picazzano, in southern Etruria, established by \insertCite{close-brooks_veii_1979;textual}{lakhesis}.
}
\examples{
data("quattrofontanili")
print(quattrofontanili)

}
\references{
\insertAllCited{}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/incidence.R
\name{im_merge}
\alias{im_merge}
\alias{im_merge.matrix}
\alias{im_merge.incidence_matrix}
\title{Merge Two Incidence Matrices}
\usage{
im_merge(obj1, obj2)

\method{im_merge}{matrix}(obj1, obj2)

\method{im_merge}{incidence_matrix}(obj1, obj2)
}
\arguments{
\item{obj1, obj2}{Two incidence matrices of any size.}
}
\value{
A single incidence matrix.
}
\description{
From two incidience matrices, create a single incidence matrix. Matrices may contain same row or column elements.
}
\examples{
data(quattrofontanili)
qf1 <- quattrofontanili[1:20, 1:40]
qf1 <- qf1[rowSums(qf1) != 0, colSums(qf1) != 0]

qf2 <- quattrofontanili[30:50, 20:60]
qf2 <- qf2[rowSums(qf2) != 0, colSums(qf2) != 0]

im_merge(qf1, qf2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/criterion.R
\name{conc_c}
\alias{conc_c}
\alias{conc_c.matrix}
\alias{conc_c.incidence_matrix}
\title{Optimality Criterion: Kendall-Doran (Column) Concentration}
\usage{
conc_c(obj)

\method{conc_c}{matrix}(obj)

\method{conc_c}{incidence_matrix}(obj)
}
\arguments{
\item{obj}{A seriated binary matrix.}
}
\value{
The measure of concentration.
}
\description{
The Kendall-Doran measure of concentration \insertCite{kendall_statistical_1963,doran_computer_1971}{lakhesis}. In a seriated matrix, this function computes the total number cells between the first and last non-zero value, column by column.
}
\examples{
data("quattrofontanili")
conc_c(quattrofontanili)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/strand.R
\name{strand_extract}
\alias{strand_extract}
\title{Strand Extract}
\usage{
strand_extract(strands)
}
\arguments{
\item{strands}{A \code{list} of class \code{strands}.}
}
\value{
A list of two matrices:
\itemize{
\item \code{Row} A matrix of the ranks of the row elements.
\item \code{Col} A matrix of the ranks of the column elements.
}
}
\description{
From a \code{list} of strands produced by \code{\link[lakhesis]{ca_procrustes_ser}}, extract two matrices containing the ranks of the rows and columns. The row/column elements are contained in the rows, and the strands are contained in the columns. NA values are entered where a given row/column element is missing from that strand.
}
\examples{
data("quattrofontanili")
data("qf_strands")
strand_extract(qf_strands)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/incidence.R
\name{im_ref}
\alias{im_ref}
\alias{im_ref.matrix}
\title{Create Reference Matrix}
\usage{
im_ref(obj)

\method{im_ref}{matrix}(obj)
}
\arguments{
\item{obj}{A matrix of size \eqn{n \times k}.}
}
\value{
A matrix of size \eqn{n \times k} with 1s along the diagonal. If \eqn{n > k}, 1s are placed from cell \eqn{(i,i)} to \eqn{(i,i+k-n)}, with 0 in all other cells.
}
\description{
Create an ideal reference matrix of well-seriated values of the same size as the input matrix.
}
\examples{
im_ref(matrix(NA, 5, 5))
im_ref(matrix(1, 7, 12))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lakhesize.R
\name{lakhesize}
\alias{lakhesize}
\alias{lakhesize.strands}
\alias{lakhesize.default}
\title{Lakhesize}
\usage{
lakhesize(strands, crit = "cor_sq", pbar = TRUE)

\method{lakhesize}{strands}(strands, crit = "cor_sq", pbar = TRUE)

\method{lakhesize}{default}(strands, crit = "cor_sq", pbar = TRUE)
}
\arguments{
\item{strands}{A \code{list} of \code{strands} class (see \code{\link[lakhesis]{strand_add}}).}

\item{crit}{The criterion used to assess the seration resulting from two strands \eqn{x} and \eqn{y}:
\itemize{
\item \code{"cor_sq"} Computes a squared correlation coefficent for the incidences of 1s as points \eqn{(i,j)}. See \code{\link[lakhesis]{cor_sq}}. Higher values are more optimal. This is the default option.
\item \code{"conc_wrc"} Computes weighted row-column concentration as the optimality criterion. See \code{\link[lakhesis]{conc_wrc}}. Lower values are more optimal.
}}

\item{pbar}{Displaying a progress bar. Default is \code{TRUE}.}
}
\value{
A \code{list} of class \code{lakhesis} containing the following:
\itemize{
\item \code{row} A seriated vector of row elements.
\item \code{col} A seriated vector of column elements
\item \code{coef}  A \code{data frame} containing the following columns:
\itemize{
\item \code{Strand} The number of the strand.
\item \code{Agreement} The measure of agreement, i.e., how well each strand accords with the consensus seriation. Using the square of Spearman's rank correlation coefficient, \eqn{\rho^2}, between each strand and the consensus ranking, agreement is computed as the product of \eqn{\rho^2} for their row and column rankings, \eqn{\rho_r^2}\eqn{\rho_c^2}.
\item \code{Criterion} Criterion of the optimality of each strand (per the \code{"crit"} option above).
}
\item \code{im_seriated} The seriated incidence matrix, of class \code{incidence_matrix}.
}
}
\description{
This function returns the row and column consensus seriation for a \code{list} object of the \code{strands} class, containing their rankings, coefficients of association, and criterion. Consensus seriation is achieved by iterative simple linear regression to handle \code{NA} vales in each strand. To initialize, a regression is performed pairwise, with every strand as the dependent \eqn{y} variate and every other strand as the independent \eqn{x} variate. The independent variate's rankings are then regressed onto \eqn{f(x) = \hat{\beta}_1 x + \hat{\beta}_0}. If \eqn{y \neq f(x)}, the mean of \eqn{y} and \eqn{f(x)} is used. Then, the values of the dependent variate and those of the regressed independent varaite are re-ranked together, to form a combined ranking, which serves as the dependent variate on the next iteration. The pair of strands is chosen which minimizes a specified optimality criterion. The process is repeated until all strands have been regressed and re-ranked into a single consensus seriation.
}
\examples{
data("qf_strands")
L <- lakhesize(qf_strands, pbar = FALSE)
# summary(L) 

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/criterion.R
\name{conc_wrc}
\alias{conc_wrc}
\alias{conc_wrc.matrix}
\alias{conc_wrc.incidence_matrix}
\title{Optimality Criterion: Weighted Row-Column Concentration}
\usage{
conc_wrc(obj)

\method{conc_wrc}{matrix}(obj)

\method{conc_wrc}{incidence_matrix}(obj)
}
\arguments{
\item{obj}{A seriated binary matrix.}
}
\value{
The weighted row-column coefficient of concentration.
}
\description{
Extends the Kendall-Doran (column) measure of concentration (see \code{\link[lakhesis]{conc_c}}) to include rows and then weights the total measure by the total sum of values in the matrix.
}
\examples{
data("quattrofontanili")
conc_wrc(quattrofontanili)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/strand.R
\name{strand_suppress}
\alias{strand_suppress}
\alias{strand_suppress.strands}
\alias{strand_suppress.default}
\title{Suppress Element from Strands}
\usage{
strand_suppress(strands, elements)

\method{strand_suppress}{strands}(strands, elements)

\method{strand_suppress}{default}(strands, elements)
}
\arguments{
\item{strands}{A \code{list} of class \code{strands}.}

\item{elements}{A vector of one or more row or column ids to suppress.}
}
\value{
A list of the strands.
}
\description{
Given a \code{list} of \code{strands} produced by correspondence analysis with Procrustes fitting (\code{\link[lakhesis]{ca_procrustes_ser}}), remove one or more row or column elements, re-seriating each strand. This generates a new list of strands that exclude the specified elements. If a resulting strand lacks sufficient points to perform correspondence analysis, that strand is deleted in the output.
}
\examples{
data("qf_strands")
strand_suppress(qf_strands, "QF II 15-16")

strand_suppress(qf_strands, c("QF II 15-16", "I", "XIV"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/strand.R
\name{strands_create}
\alias{strands_create}
\alias{strands_create.list}
\title{Create List of Strands}
\usage{
strands_create(strands)

\method{strands_create}{list}(strands)
}
\arguments{
\item{strands}{A \code{list} of strands.}
}
\value{
A \code{list} of class \code{strands}.
}
\description{
Given one or more individual strand objects, create a single \code{list} of class \code{strands}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/criterion.R
\name{spearman_sq}
\alias{spearman_sq}
\alias{spearman_sq.numeric}
\title{Spearman Correlation Squared}
\usage{
spearman_sq(r1, r2)

\method{spearman_sq}{numeric}(r1, r2)
}
\arguments{
\item{r1, r2}{Two vectors of paired ranks.}
}
\value{
The square of Spearman's rank correlation coefficient with NA values removed.
}
\description{
The square of Spearman's rank correlation coefficient applied to two rankings \insertCite{spearman_proof_1904}{lakhesis}. Rows with \code{NA} values are automatically removed.
}
\examples{
# e.g., for two partial seriations:
x <- c(1, 2, 3, 4, NA, 5, 6, NA, 7.5, 7.5, 9)
y <- c(23, 17, 19, NA, 21, 22, 25, 26, 27, 36, 32)
spearman_sq(x, y)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/procrustes.R
\name{ca_procrustes}
\alias{ca_procrustes}
\alias{ca_procrustes.matrix}
\alias{ca_procrustes.incidence_matrix}
\title{Correspondence Analysis with Procrustes Fitting}
\usage{
ca_procrustes(obj, symmetric = TRUE)

\method{ca_procrustes}{matrix}(obj, symmetric = TRUE)

\method{ca_procrustes}{incidence_matrix}(obj, symmetric = TRUE)
}
\arguments{
\item{obj}{An incidence matrix of size n x k.}

\item{symmetric}{Whether to use standard scores for both rows and columns. Default is \code{TRUE}. Setting this to \code{FALSE} will result in a asymmetric map of standard scores for rows and principal scores for columns.}
}
\value{
A \code{list} object of class \code{strand} containing the following:
\itemize{
\item \code{ref} The Procrustes-fit coordinates of the scores of the reference seriation.
\item \code{x} The coordinates of the row standard scores of the data.
\item \code{y} The coordinates of the column principal scores of the data.
\item \code{x_pr} The Procrustes-fit coordinates of the row standard scores of the data.
\item \code{y_pr} The Procrustes-fit coordinates of the column column scores of the data.
}
}
\description{
Fit scores of correspondence analysi on an incidence matrix to those produced by reference matrix which contain an ideal seriation using a Procrustes method (on the reference matrix, see \code{\link[lakhesis]{im_ref}}).  Rotation is determined by minimizing Euclidean distance from each row score to the nearest reference row score. Correspondence analysis is performed using the \code{\link[ca]{ca}} package \insertCite{nenadic_correspondence_2007}{lakhesis}.
}
\examples{
data("quattrofontanili")
s <- ca_procrustes(quattrofontanili)
# print(s)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/app.R
\name{LC}
\alias{LC}
\title{Lakhesis Calculator}
\usage{
LC()
}
\value{
Opens the Lakhesis Calculator.
}
\description{
Launch Lakhesis Calculator, a graphical interface to explore binary matrices via correspondence analysis, select potentially well-seriated sequences, and perform consensus seriation. Interface is made with \code{ggplot2}, \code{shiny}, \code{shinydashboard}, and \code{bslib} \insertCite{wickham_ggplot2:_2016,chang_shiny_2024,chang_shinydashboard_2021,sievert_bslib_2024}{lakhesis}.
}
\details{
Input is done in the calculator, via a "long" format a two-column \code{.csv} file giving pairs of row and column incidences. See \code{\link[lakhesis]{im_read_csv}} for details. Conversion of a pre-existing incidence matrix to long format can be performed with \code{\link[lakhesis]{im_long}}.

Results can be downloaded from the calculator as an \code{.rds} file containing a \code{list} of the following:
\itemize{
\item \code{consensus} The consensus seriations, diagnostic coefficients of agreement and optimality criterion, and seriated incidence matrix.
(\code{\link[lakhesis]{lakhesize}}).
\item \code{strands} The strands selected by the investigator.
}
}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/procrustes.R
\name{ca_procrustes_ser}
\alias{ca_procrustes_ser}
\alias{ca_procrustes_ser.incidence_matrix}
\alias{ca_procrustes_ser.matrix}
\title{Seriate Procrustes-Fit CA Scores}
\usage{
ca_procrustes_ser(obj, projection = "curve", samples = 10^5, symmetric = TRUE)

\method{ca_procrustes_ser}{incidence_matrix}(obj, projection = "curve", samples = 10^5, symmetric = TRUE)

\method{ca_procrustes_ser}{matrix}(obj, projection = "curve", samples = 10^5, symmetric = TRUE)
}
\arguments{
\item{obj}{An incidence matrix of size n x k.}

\item{projection}{Which projection to use:
\itemize{
\item \code{ca1} The first axis for CA scores.
\item \code{ca2} The second axis for CA scores.
\item \code{procrustes1} The first axis of Procrustes-fit CA scores.
\item \code{procrustes2} The second axis of Procrustes-fit CA scores.
\item \code{curve} The reference curve of an ideal seriation, using Procrustes fitting (this is the default).
}}

\item{samples}{Number of samples to use for plotting points along polynomial curve. Default is \code{10^5}.}

\item{symmetric}{Whether to use standard scores for both rows and columns. Default is \code{TRUE}. Setting this to \code{FALSE} will result in a asymmetric map of standard scores for rows and principal scores for columns.}
}
\value{
A \code{list} of class \code{strand} containing the following:.
\itemize{
\item \code{$dat} A data frame with the following columns:
\itemize{
\item \code{Procrustes1, Procrustes2} The location of the point on the biplot after fitting.
\item \code{CurveIndex} The orthogonal projection of the point onto the reference curve, given as the index of the point sampled along \eqn{y = \beta_2 x^2 + \beta_0}.
\item \code{Distance} The squared Euclidean distance of the point to the nearest point on the reference curve.
\item \code{Rank} The ranking of the row or column, a range of \verb{1:nrow`` and }1:ncol``.
\item \code{Type} Either \code{row} or \code{col}.
\item \code{sel} Data frame column used in \code{shiny} app to indicate whether point is selected in biplot/curve projection.
}
\item \code{$im_seriated} The seriated incidence matrix, of class \code{incidence_matrix}.
}
}
\description{
Obtain a ranking of row and column scores projected onto a reference curve of an ideal seriation (row and column scores are ranked separately). Scores of correspondence analysis have been fit to those produced by reference matrix contain an ideal seriation using a Procrustes method, projecting them. Rotation is determined by minimizing Euclidean distance from each row score to the nearest reference row score. Correspondence analysis is performed using the \code{\link[ca]{ca}} package \insertCite{nenadic_correspondence_2007}{lakhesis}.
}
\examples{
data("quattrofontanili")
s <- ca_procrustes_ser(quattrofontanili)
# print(s)
# summary(s)

}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/strand.R
\name{strand_add}
\alias{strand_add}
\alias{strand_add.strand}
\title{Add Strand to List of Strands}
\usage{
strand_add(strand, strands)

\method{strand_add}{strand}(strand, strands)
}
\arguments{
\item{strand}{An object of class \code{strand} returned by \code{\link[lakhesis]{ca_procrustes_ser}}.}

\item{strands}{A \code{list} of strands.}
}
\value{
A \code{list} of class \code{strands}.
}
\description{
Given a list of strands, remove a row or column element and re-run seriation by correspondence analysis with Procrustes fitting (\code{\link[lakhesis]{ca_procrustes_ser}}) to generate a new list of strands that exclude the specified elements. If the resulting strand lacks sufficient points to perform correspondence analysis, that strand is deleted in the output.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{qf_strands}
\alias{qf_strands}
\title{Quattro Fontanili - Strands}
\format{
A \code{strands} object containing strands output by \code{\link[lakhesis]{ca_procrustes}}.
}
\usage{
data("qf_strands")
}
\description{
Three seriated strands selected from \code{\link[lakhesis]{quattrofontanili}} data, identified by the package author as an example for the documentation of functions.
}
\examples{
data("qf_strands")
print(qf_strands)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/strand.R
\name{strand_create}
\alias{strand_create}
\alias{strand_create.matrix}
\alias{strand_create.incidence_matrix}
\title{Create Strand Object from Seriated Incidence Matrix}
\usage{
strand_create(obj, method = NULL)

\method{strand_create}{matrix}(obj, method = NULL)

\method{strand_create}{incidence_matrix}(obj, method = NULL)
}
\arguments{
\item{obj}{A \code{list} of strands.}

\item{method}{The method used to create the strand (optional).}
}
\value{
A \code{list} of class \code{strands}.
}
\description{
Given a seriated incidence matrix with unique row and column names, create a \code{strand} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/criterion.R
\name{element_eval}
\alias{element_eval}
\alias{element_eval.matrix}
\alias{element_eval.incidence_matrix}
\title{Evaluating Element Fit}
\usage{
element_eval(obj)

\method{element_eval}{matrix}(obj)

\method{element_eval}{incidence_matrix}(obj)
}
\arguments{
\item{obj}{A seriated binary matrix.}
}
\value{
A \code{list} containing results in data frames for row and column elements:
\itemize{
\item \code{RowFit} a data frame containing
\itemize{
\item \code{id} Row element
\item \code{p.val} \eqn{p} values of the row elements
}
\item \code{ColFit} a data frame containing
\itemize{
\item \code{id} Column element
\item \code{p.val} \eqn{p} values of the column elements
}
}
}
\description{
Performs a goodness-of-fit test on individual row and column elements using deviance, using a quadratic-logistic model to fit row and column occurrences. In the case of perfect separation of 0/1 values, an \code{NA} value is assigned. Results are reported as \eqn{p} values for each row and column.
}
\examples{
data("quattrofontanili")
element_eval(quattrofontanili)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/incidence.R
\name{im_read_csv}
\alias{im_read_csv}
\title{Read csv File to Incidence Matrix}
\usage{
im_read_csv(
  filename,
  header = FALSE,
  characterencoding = "iso-8859-1",
  remove.hapax = FALSE
)
}
\arguments{
\item{filename}{The filename to uploaded (must be in \code{.csv} format).}

\item{header}{If the \code{.csv} file contains a header. Default is \code{FALSE}.}

\item{characterencoding}{File encoding as used by \code{\link[readr]{locale}}, which depends on the \code{csv} file. Default is \code{"iso-8859-1"} but \code{"utf-8"} is also recommended.}

\item{remove.hapax}{Remove any row or column which has a sum of 1 (i.e., is only attested once), since they do not directly contribute to the result of the seriation. Default is \code{FALSE}.}
}
\value{
A matrix of binary values (0 = row/column occurrence is absence; 1 = row/column occurrence is present).
}
\description{
Wrapper around the \code{\link[readr]{read_csv}} function from the \code{readr} package \insertCite{wickham_readr_2024}{lakhesis}. Read a \code{.csv} file in which the first column represents row elements and the second column represents column elements, and convert it into an incidence matrix.
}
\references{
\insertAllCited{}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/incidence.R
\name{im_long}
\alias{im_long}
\alias{im_long.matrix}
\alias{im_long.incidence_matrix}
\title{Convert Incidence Matrix to Pairs (Long Format)}
\usage{
im_long(obj)

\method{im_long}{matrix}(obj)

\method{im_long}{incidence_matrix}(obj)
}
\arguments{
\item{obj}{An incidence matrix.}
}
\value{
A data frame of two columns (row and column of the incidence matrix), in which row of the data frame represents a pair of an
}
\description{
Take an incidence matrix and convert it to a data frame of two columns, where the first column represents the row elements of the incidence matrix and the second column represents the column elements of the incidence matrix. Each row pair represents the incidence (or occurrence) of that row and column element together.
}
\examples{
data(quattrofontanili)
qf <- im_long(quattrofontanili)

# to export for uploading into the Lakhesis Calculator, use write.table() to 
# remove both row and column names:

# write.table(qf, file = 'qf.csv', row.names = FALSE, col.names = FALSE, sep = ",")

}
